@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Graphics state management for pdfwrite driver */
 #include "math_.h"
 #include "string_.h"
@@ -234,6 +236,8 @@ pdf_reset_graphics_old(gx_device_pdf * pdev)
     }
     pdev->fill_overprint = false;
     pdev->stroke_overprint = false;
+    pdev->remap_fill_color = false;
+    pdev->remap_stroke_color = false;
     pdf_reset_text(pdev);
 }
 
@@ -417,8 +421,14 @@ pdf_reset_color(gx_device_pdf * pdev, const gs_imager_state * pis,
                 else if (pdc->type == &gx_dc_pure_masked) {
                     code = pdf_put_uncolored_pattern(pdev, pdc, pcs,
                                 ppscc, pis->have_pattern_streams, &pres);
-                    if (code < 0 || pres == 0)
+                    if (code < 0 || pres == 0) {
+                        /* replaced a pattern with a flat fill, but we still
+                         * need to change the 'saved' colour or we will
+                         * get out of step with the PDF content.
+                         */
+                        *psc = temp;
                         return code;
+                    }
                     if (pis->have_pattern_streams)
                         code = pdf_write_ccolor(pdev, pis, pcc);
                 } else if (pdc->type == &gx_dc_pattern2) {
@@ -878,6 +888,18 @@ pdf_write_spot_function(gx_device_pdf *pdev, const gx_ht_order *porder,
     gs_free_object(mem, values, "pdf_write_spot_function");
     return code;
 }
+
+/* if (memcmp(order.levels, porder->levels, order.num_levels * sizeof(*order.levels))) */ 
+static int
+compare_gx_ht_order_levels(const gx_ht_order *order1, const gx_ht_order *order2) {
+  int i;
+  for (i=0;  i<order1->num_levels;  i++) {
+    if (order1->levels[i] != order2->levels[i])
+      return(1);
+  }
+  return(0);
+}
+
 static int
 pdf_write_spot_halftone(gx_device_pdf *pdev, const gs_spot_halftone *psht,
                         const gx_ht_order *porder, long *pid)
@@ -916,8 +938,7 @@ pdf_write_spot_halftone(gx_device_pdf *pdev, const gs_spot_halftone *psht,
             if (code < 0)
                 continue;
             /* Compare the bits and levels arrays. */
-            if (memcmp(order.levels, porder->levels,
-                       order.num_levels * sizeof(*order.levels)))
+            if (compare_gx_ht_order_levels(&order,porder))
                 continue;
             if (memcmp(order.bit_data, porder->bit_data,
                        order.num_bits * porder->procs->bit_data_elt_size))
@@ -1469,7 +1490,7 @@ pdf_prepare_drawing(gx_device_pdf *pdev, const gs_imager_state *pis,
                 return code;
         }
         if (pdev->params.TransferFunctionInfo == tfi_Preserve &&
-            !pdev->PDFX && !pdev->PDFA
+            !pdev->PDFX && pdev->PDFA == 0
             ) {
             code = pdf_update_transfer(pdev, pis, trs);
             if (code < 0)
