@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* FontDescriptor implementation for pdfwrite */
 #include "math_.h"
 #include "memory_.h"
@@ -19,6 +21,7 @@
 #include "gserrors.h"
 #include "gsrect.h"		/* for rect_merge */
 #include "gscencs.h"
+#include "gxfcopy.h"
 #include "gdevpdfx.h"
 #include "gdevpdfo.h"		/* for object->written */
 #include "gdevpdtb.h"
@@ -155,18 +158,27 @@ pdf_font_descriptor_common_id(const pdf_font_descriptor_common_t *pfdc)
 /* Write the common part of a FontDescriptor, aside from the final >>. */
 static int
 write_FontDescriptor_common(gx_device_pdf *pdev,
-                            const pdf_font_descriptor_common_t *pfd)
+                            const pdf_font_descriptor_common_t *pfd, bool embed)
 {
     stream *s;
     int code;
     param_printer_params_t params;
     printer_param_list_t rlist;
     gs_param_list *const plist = (gs_param_list *)&rlist;
+    char *base14_name = NULL;
 
     pdf_open_separate(pdev, pdf_font_descriptor_common_id(pfd), resourceFontDescriptor);
     s = pdev->strm;
     stream_puts(s, "<</Type/FontDescriptor/FontName");
-    pdf_put_name(pdev, pfd->values.FontName.data, pfd->values.FontName.size);
+    if (!embed) {
+        base14_name = (char *)pdf_find_base14_name(pfd->values.FontName.data, pfd->values.FontName.size);
+        if(base14_name)
+            pdf_put_name(pdev, (byte *)base14_name, strlen(base14_name));
+        else
+            pdf_put_name(pdev, pfd->values.FontName.data, pfd->values.FontName.size);
+    } else
+        pdf_put_name(pdev, pfd->values.FontName.data, pfd->values.FontName.size);
+
     pdf_write_font_bbox(pdev, &pfd->values.FontBBox);
     params = param_printer_params_default;
     code = s_init_param_printer(&rlist, &params, s);
@@ -194,6 +206,9 @@ write_FontDescriptor_common(gx_device_pdf *pdev,
         int Flags = pfd->values.Flags;
         pdf_font_descriptor_t defaults;
 
+        if (base14_name)
+            Flags |=FONT_USES_STANDARD_ENCODING;
+
         code = param_write_int(plist, "Flags", &Flags);
         if (code < 0)
             return code;
@@ -243,6 +258,27 @@ pdf_font_descriptor_alloc(gx_device_pdf *pdev, pdf_font_descriptor_t **ppfd,
     return 0;
 }
 
+int pdf_font_descriptor_free(gx_device_pdf *pdev, pdf_resource_t *pres)
+{
+    pdf_font_descriptor_t *pfd = (pdf_font_descriptor_t *)pres;
+    pdf_base_font_t *pbfont = pfd->base_font;
+    gs_font *copied = (gs_font *)pbfont->copied;
+
+    gs_free_copied_font(copied);
+    if (pbfont && pbfont->font_name.size) {
+        gs_free_string(pdev->memory, pbfont->font_name.data, pbfont->font_name.size, "Free BaseFont FontName string");
+        pbfont->font_name.data = (byte *)0L;
+        pbfont->font_name.size = 0;
+    }
+    if (pbfont)
+        gs_free_object(cos_object_memory(pres->object), pbfont, "Free base font from FontDescriptor)");
+    if (pres->object) {
+        gs_free_object(cos_object_memory(pres->object), pres->object, "free FontDescriptor object");
+        pres->object = NULL;
+    }
+    return 0;
+}
+
 /* Get the object ID of a FontDescriptor. */
 long
 pdf_font_descriptor_id(const pdf_font_descriptor_t *pfd)
@@ -514,7 +550,7 @@ pdf_compute_font_descriptor(gx_device_pdf *pdev, pdf_font_descriptor_t *pfd)
     if (!(desc.Flags & FONT_IS_SYMBOLIC)) {
         desc.Flags |= FONT_IS_ADOBE_ROMAN; /* required if not symbolic */
         desc.XHeight = (int)x_height;
-        if (!small_present && (!pdev->PDFA || bfont->FontType != ft_TrueType))
+        if (!small_present && (!pdev->PDFA != 0 || bfont->FontType != ft_TrueType))
             desc.Flags |= FONT_IS_ALL_CAPS;
         desc.CapHeight = cap_height;
         /*
@@ -565,10 +601,10 @@ pdf_compute_font_descriptor(gx_device_pdf *pdev, pdf_font_descriptor_t *pfd)
     desc.Descent = desc.FontBBox.p.y;
     if (!(desc.Flags & (FONT_IS_SYMBOLIC | FONT_IS_ALL_CAPS)) &&
         (small_descent > desc.Descent / 3 || desc.XHeight > small_height * 0.9) &&
-        (!pdev->PDFA || bfont->FontType != ft_TrueType)
+        (!pdev->PDFA != 0 || bfont->FontType != ft_TrueType)
         )
         desc.Flags |= FONT_IS_SMALL_CAPS;
-    if (fixed_width > 0 && (!pdev->PDFA || bfont->FontType != ft_TrueType)) {
+    if (fixed_width > 0 && (!pdev->PDFA != 0 || bfont->FontType != ft_TrueType)) {
         desc.Flags |= FONT_IS_FIXED_WIDTH;
         desc.AvgWidth = desc.MaxWidth = desc.MissingWidth = fixed_width;
     }
@@ -625,9 +661,11 @@ pdf_write_FontDescriptor(gx_device_pdf *pdev, pdf_resource_t *pres)
     case ft_CID_encrypted:
     case ft_CID_TrueType:
         if (pdf_do_subset_font(pdev, pfd->base_font, pfd->common.rid)) {
-            code = pdf_write_CIDSet(pdev, pfd->base_font, &cidset_id);
-            if (code < 0)
-                return code;
+            if (pdev->PDFA < 2) {
+                code = pdf_write_CIDSet(pdev, pfd->base_font, &cidset_id);
+                if (code < 0)
+                    return code;
+            }
         }
     default:
         break;
@@ -646,7 +684,7 @@ pdf_write_FontDescriptor(gx_device_pdf *pdev, pdf_resource_t *pres)
             )
             fd.values.Flags =
                 (fd.values.Flags & ~(FONT_IS_ADOBE_ROMAN)) | FONT_IS_SYMBOLIC;
-        code = write_FontDescriptor_common(pdev, &fd);
+        code = write_FontDescriptor_common(pdev, &fd, pfd->embed);
     }
     if (code < 0)
         return code;
@@ -682,6 +720,7 @@ pdf_write_FontDescriptor(gx_device_pdf *pdev, pdf_resource_t *pres)
     pfd->common.object->written = true;
     {	const cos_object_t *pco = (const cos_object_t *)pdf_get_FontFile_object(pfd->base_font);
         if (pco != NULL) {
+            pprintld1(s, "%%BeginResource: file (PDF FontFile obj_%ld)\n", pco->id);
             code = COS_WRITE_OBJECT(pco, pdev, resourceNone);
             if (code < 0)
                 return code;
@@ -728,8 +767,8 @@ pdf_convert_truetype_font_descriptor(gx_device_pdf *pdev, pdf_font_resource_t *p
     /* Save the simple font descriptor data because CID font data overlap them. */
     int FirstChar = pdfont->u.simple.FirstChar, LastChar = pdfont->u.simple.LastChar;
     pdf_encoding_element_t *Encoding = pdfont->u.simple.Encoding;
-    int length_CIDSet = (pbfont->num_glyphs > LastChar ? (pbfont->num_glyphs + 7) / 8 : (LastChar + 7 / 8));
-    int length_CIDToGIDMap = (pbfont->num_glyphs > LastChar ? pbfont->num_glyphs * sizeof(ushort) : LastChar * sizeof(ushort));
+    int length_CIDSet = (pbfont->num_glyphs > LastChar ? (pbfont->num_glyphs + 7) / 8 : ((LastChar + 1) + 7 / 8));
+    int length_CIDToGIDMap = (pbfont->num_glyphs > LastChar ? pbfont->num_glyphs * sizeof(ushort) : (LastChar + 1) * sizeof(ushort));
 
     pfd->FontType = ft_CID_TrueType;
     pdfont->u.simple.Encoding = NULL; /* Drop due to overlapping against a garbager problem. */
@@ -743,16 +782,25 @@ pdf_convert_truetype_font_descriptor(gx_device_pdf *pdev, pdf_font_resource_t *p
     if (pdfont->u.cidfont.CIDToGIDMap == NULL)
         return_error(gs_error_VMerror);
     memset(pdfont->u.cidfont.CIDToGIDMap, 0, length_CIDToGIDMap);
-    for (ch = FirstChar; ch <= LastChar; ch++) {
-        if (Encoding[ch].glyph != GS_NO_GLYPH) {
+    if(pdev->PDFA == 1) {
+        for (ch = FirstChar; ch <= LastChar; ch++) {
+            if (Encoding[ch].glyph != GS_NO_GLYPH) {
+                gs_glyph glyph = pfont->procs.encode_char(pfont, ch, GLYPH_SPACE_INDEX);
+
+                pbfont->CIDSet[ch / 8] |= 0x80 >> (ch % 8);
+                pdfont->u.cidfont.CIDToGIDMap[ch] = glyph - GS_MIN_GLYPH_INDEX;
+            }
+        }
+        /* Set the CIDSet bit for CID 0 (the /.notdef) which must always be present */
+        pbfont->CIDSet[0] |= 0x80;
+    } else {
+        for (ch = 0; ch <= pbfont->num_glyphs; ch++) {
             gs_glyph glyph = pfont->procs.encode_char(pfont, ch, GLYPH_SPACE_INDEX);
 
             pbfont->CIDSet[ch / 8] |= 0x80 >> (ch % 8);
             pdfont->u.cidfont.CIDToGIDMap[ch] = glyph - GS_MIN_GLYPH_INDEX;
         }
     }
-    /* Set the CIDSet bit for CID 0 (the /.notdef) which must always be present */
-    pbfont->CIDSet[0] |= 0x80;
     pbfont->CIDSetLength = length_CIDSet;
     pdfont->u.cidfont.CIDToGIDMapLength = length_CIDToGIDMap / sizeof(ushort);
     pdfont->u.cidfont.Widths2 = NULL;
