@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Image processing streams for PostScript and PDF writers */
 #include "gx.h"
 #include "memory_.h"
@@ -34,7 +36,7 @@ int
 s_1248_init(stream_1248_state *ss, int Columns, int samples_per_pixel)
 {
     ss->samples_per_row = Columns * samples_per_pixel;
-    return ss->template->init((stream_state *)ss);
+    return ss->templat->init((stream_state *)ss);
 }
 
 /* Initialize the state. */
@@ -517,13 +519,13 @@ const stream_template s_IE_template = {
     0 /* NULL */, s_IE_set_defaults
 };
 
-#if 0
+#ifdef DEPRECATED_906
 
 /* Test code */
 void
 test_IE(void)
 {
-    const stream_template *const template = &s_IE_template;
+    const stream_templat *const template = &s_IE_template;
     stream_IE_state state;
     stream_state *const ss = (stream_state *)&state;
     static const float decode[6] = {1, 0, 1, 0, 1, 0};
@@ -540,7 +542,7 @@ test_IE(void)
     byte table[3 * 5];
     int n;
 
-    template->set_defaults(ss);
+    templat->set_defaults(ss);
     state.BitsPerComponent = 4;
     state.NumComponents = 3;
     state.Width = 3;
@@ -561,7 +563,7 @@ test_IE(void)
         memset(table, 0xcc, sizeof(table));
         memset(out, 0xff, sizeof(out));
         dprintf1("processing %d bytes\n", n);
-        status = template->process(ss, &r, &w, true);
+        status = templat->process(ss, &r, &w, true);
         dprintf3("%d bytes read, %d bytes written, status = %d\n",
                  (int)(r.ptr + 1 - in), (int)(w.ptr + 1 - out), status);
         debug_dump_bytes(table, table + sizeof(table), "table");
