@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Utilities for "vector" devices */
 #include "math_.h"
 #include "memory_.h"
@@ -101,6 +103,7 @@ gdev_vector_dopath(gx_device_vector *vdev, const gx_path * ppath,
     sw:
         if (type & gx_path_type_optimize) {
         opt:
+            /* RJW: We fail to optimize gaptos */
             if (pe_op == gs_pe_lineto) {
                 if (!incomplete_line) {
                     line_end = vs[0];
@@ -173,6 +176,7 @@ gdev_vector_dopath(gx_device_vector *vdev, const gx_path * ppath,
             }
             goto draw;
         case gs_pe_lineto:
+        case gs_pe_gapto:
             if (need_moveto) {	/* see gs_pe_moveto case */
                 code = gdev_vector_dopath_segment(&state, gs_pe_moveto,
                                                   &line_start);
@@ -278,7 +282,7 @@ gdev_vector_open_file_options(gx_device_vector * vdev, uint strmbuf_size,
         code = gx_device_open_output_file((gx_device *)vdev, vdev->fname,
                                           binary, false, &vdev->file);
     }
-    if (code >= 0) {
+    if ((code >= 0) && (dev_proc(vdev, get_profile) != NULL)) {
         code = dev_proc(vdev, get_profile)((gx_device *)vdev, &icc_struct);
     }
 
@@ -642,6 +646,7 @@ gdev_vector_dopath_segment(gdev_vector_dopath_state_t *state, int pe_op,
             state->prev = vp[0];
             break;
         case gs_pe_lineto:
+        case gs_pe_gapto: /* FIXME */
             code = gs_point_transform_inverse(fixed2float(vs[0].x),
                                        fixed2float(vs[0].y), pmat, &vp[0]);
             if (code < 0)
@@ -943,6 +948,7 @@ gdev_vector_get_params(gx_device * dev, gs_param_list * plist)
     int code = gx_default_get_params(dev, plist);
     int ecode;
     gs_param_string ofns;
+    bool bool_true = 1;
 
     if (code < 0)
         return code;
@@ -951,6 +957,8 @@ gdev_vector_get_params(gx_device * dev, gs_param_list * plist)
         ofns.persistent = false;
     if ((ecode = param_write_string(plist, "OutputFile", &ofns)) < 0)
         return ecode;
+    if ((ecode = param_write_bool(plist, "HighLevelDevice", &bool_true)) < 0)
+        return ecode;
     return code;
 }
 
@@ -962,6 +970,12 @@ gdev_vector_put_params(gx_device * dev, gs_param_list * plist)
     int code;
     gs_param_name param_name;
     gs_param_string ofns;
+    bool open = dev->is_open, HighLevelDevice;
+
+
+    code = param_read_bool(plist, (param_name = "HighLevelDevice"), &HighLevelDevice);
+    if (code < 0)
+        return code;
 
     switch (code = param_read_string(plist, (param_name = "OutputFile"), &ofns)) {
         case 0:
@@ -979,18 +993,14 @@ gdev_vector_put_params(gx_device * dev, gs_param_list * plist)
                 /* The new name is the same as the old name.  Do nothing. */
                 ofns.data = 0;
                 break;
-            } else if (dev->LockSafetyParams ||
-                        (dev->is_open && vdev->strm != 0 &&
-                       stell(vdev->strm) != 0)
-                       )
-                ecode = (dev->LockSafetyParams) ? gs_error_invalidaccess :
-                                gs_error_rangecheck;
-            else
-                break;
-            goto ofe;
+            } else if (dev->LockSafetyParams) {
+                    ecode = gs_error_invalidaccess;
+                    goto ofe;
+            }
+            break;
         default:
             ecode = code;
-          ofe:param_signal_error(plist, param_name, ecode);
+ofe:        param_signal_error(plist, param_name, ecode);
         case 1:
             ofns.data = 0;
             break;
@@ -999,8 +1009,6 @@ gdev_vector_put_params(gx_device * dev, gs_param_list * plist)
     if (ecode < 0)
         return ecode;
     {
-        bool open = dev->is_open;
-
         /* Don't let gx_default_put_params close the device. */
         dev->is_open = false;
         code = gx_default_put_params(dev, plist);
@@ -1010,18 +1018,35 @@ gdev_vector_put_params(gx_device * dev, gs_param_list * plist)
         return code;
 
     if (ofns.data != 0) {
+        /* If ofns.data is not NULL, then we have a different file name */
         memcpy(vdev->fname, ofns.data, ofns.size);
         vdev->fname[ofns.size] = 0;
-        if (vdev->file != 0) {
-            gx_device_bbox *bbdev = vdev->bbox_device;
+        if (dev->is_open && vdev->strm != 0 && stell(vdev->strm) != 0) {
+            /* we want to close and re-open the device so we can change the file */
+            ecode = gs_closedevice(dev);
+            if (ecode < 0) {
+                param_signal_error(plist, param_name, ecode);
+                return ecode;    /* THIS MAY CAUSE PROBLEMS SINCE THE DEVICE MAY BE CLOSED */
+            }
+            if (vdev->file != 0) {
+                gx_device_bbox *bbdev = vdev->bbox_device;
 
-            vdev->bbox_device = 0; /* don't let it be freed */
-            code = gdev_vector_close_file(vdev);
-            vdev->bbox_device = bbdev;
-            if (code < 0)
-                return code;
+                vdev->bbox_device = 0; /* don't let it be freed */
+                code = gdev_vector_close_file(vdev);
+                vdev->bbox_device = bbdev;
+                if (code < 0)
+                    return code;
+            }
+            ecode = gs_opendevice(dev);    /* opendevice is expected to open the new file */
+            if (ecode < 0) {
+                param_signal_error(plist, param_name, ecode);
+                return ecode;
+            }
+        }
+        /* device is open and hasn't written data yet, so open the file */
+        else if (dev->is_open) {
             return gdev_vector_open_file_options(vdev, vdev->strmbuf_size,
-                                                 vdev->open_options);
+                              vdev->open_options);
         }
     }
     return 0;
