@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id$ */
+
 /* Gimp (XCF) export device, supporting DeviceN color models. */
 
 #include "math_.h"
@@ -377,9 +379,7 @@ cmyk_cs_to_spotn_cm(gx_device * dev, frac c, frac m, frac y, frac k, frac out[])
         in[2] = frac2ushort(y);
         in[3] = frac2ushort(k);
 
-        gscms_transform_color(link, &(in[0]),
-                        &(tmp[0]), 2, NULL);
-
+        gscms_transform_color(dev, link, &(in[0]), &(tmp[0]), 2);
         for (i = 0; i < outn; i++)
             out[i] = ushort2frac(tmp[i]);
         for (; i < n + 4; i++)
@@ -423,8 +423,7 @@ rgb_cs_to_spotn_cm(gx_device * dev, const gs_imager_state *pis,
         in[1] = frac2ushort(g);
         in[2] = frac2ushort(b);
 
-        gscms_transform_color(link, &(in[0]),
-                        &(tmp[0]), 2, NULL);
+        gscms_transform_color(dev, link, &(in[0]), &(tmp[0]), 2);
 
         for (i = 0; i < outn; i++)
             out[i] = ushort2frac(tmp[i]);
@@ -491,14 +490,15 @@ static gx_color_index
 xcf_encode_color(gx_device *dev, const gx_color_value colors[])
 {
     int bpc = ((xcf_device *)dev)->bitspercomponent;
-    int drop = sizeof(gx_color_value) * 8 - bpc;
     gx_color_index color = 0;
     int i = 0;
     int ncomp = dev->color_info.num_components;
+    COLROUND_VARS;
 
+    COLROUND_SETUP(bpc);
     for (; i<ncomp; i++) {
         color <<= bpc;
-        color |= (colors[i] >> drop);
+        color |= COLROUND_ROUND(colors[i]);
     }
     return (color == gx_no_color_index ? color ^ 1 : color);
 }
@@ -510,13 +510,14 @@ static int
 xcf_decode_color(gx_device * dev, gx_color_index color, gx_color_value * out)
 {
     int bpc = ((xcf_device *)dev)->bitspercomponent;
-    int drop = sizeof(gx_color_value) * 8 - bpc;
     int mask = (1 << bpc) - 1;
     int i = 0;
     int ncomp = dev->color_info.num_components;
+    COLDUP_VARS;
 
+    COLDUP_SETUP(bpc);
     for (; i<ncomp; i++) {
-        out[ncomp - i - 1] = (color & mask) << drop;
+        out[ncomp - i - 1] = COLDUP_DUP(color & mask);
         color >>= bpc;
     }
     return 0;
@@ -769,7 +770,7 @@ bpc_to_depth(int ncomp, int bpc)
     if (ncomp <=4 && bpc <= 8)
         return depths[ncomp -1][bpc-1];
     else
-        return (ncomp * bpc + 7) & 0xf8;
+        return (ncomp * bpc + 7) & ~7;
 }
 
 #define BEGIN_ARRAY_PARAM(pread, pname, pa, psize, e)\
@@ -1273,7 +1274,7 @@ xcf_shuffle_to_tile(xcf_write_ctx *xc, byte **tile_data, const byte *row,
 }
 
 static void
-xcf_icc_to_tile(xcf_write_ctx *xc, byte **tile_data, const byte *row,
+xcf_icc_to_tile(gx_device_printer *pdev, xcf_write_ctx *xc, byte **tile_data, const byte *row,
                     int y, gcmmhlink_t link)
 {
     int tile_j = y / TILE_HEIGHT;
@@ -1303,8 +1304,9 @@ xcf_icc_to_tile(xcf_write_ctx *xc, byte **tile_data, const byte *row,
                    understand what is going on in the loop
                    with the 255^row[row_idx++] operation */
 
-            gscms_transform_color(link, &(row[row_idx]),
-                            &(base_ptr[base_idx]), 1, NULL);
+            gscms_transform_color((gx_device*) pdev, link, 
+                                  (void *) (&(row[row_idx])),
+                                   &(base_ptr[base_idx]), 1);
 
             for (plane_idx = 0; plane_idx < n_extra_channels; plane_idx++)
                 extra_ptr[plane_idx * extra_stride + x] = 255 ^ row[row_idx++];
@@ -1349,7 +1351,7 @@ xcf_write_image_data(xcf_write_ctx *xc, gx_device_printer *pdev)
             if (link == NULL)
                 xcf_shuffle_to_tile(xc, tile_data, row, y);
             else
-                xcf_icc_to_tile(xc, tile_data, row, y, link);
+                xcf_icc_to_tile(pdev, xc, tile_data, row, y, link);
         }
         for (tile_i = 0; tile_i < xc->n_tiles_x; tile_i++) {
             int tile_idx = tile_j * xc->n_tiles_x + tile_i;
