@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /*
  * Generate a header file (arch.h) with parameters
  * reflecting the machine architecture and compiler characteristics.
@@ -75,10 +77,33 @@ ilog2(int n)
     return i;
 }
 
+static int
+copy_existing_file(FILE *f, char *infname)
+{
+    FILE *in = fopen(infname, "r");
+    char buffer[1024];
+    size_t l;
+
+    if (in == NULL) {
+        fprintf(stderr, "genarch.c: can't open %s for reading\n", infname);
+        fclose(f);
+        return exit_FAILED;
+    }
+    while (!feof(in)) {
+        l = fread(buffer, 1, 1024, in);
+        if (l > 0)
+            fwrite(buffer, 1, l, f);
+    }
+    fclose(in);
+    fclose(f);
+
+    return exit_OK;
+}
+
 int
 main(int argc, char *argv[])
 {
-    char *fname = argv[1];
+    char *fname;
     long one = 1;
     struct {
         char c;
@@ -119,12 +144,25 @@ main(int argc, char *argv[])
         long l;
     } f0, f1, fm1;
     int floats_are_IEEE;
-    FILE *f = fopen(fname, "w");
+    FILE *f;
 
+    if (argc < 2 || argc > 3) {
+        fprintf(stderr, "genarch: Invalid invocation\n"
+                "genarch <output-file> [ <existing-config-file> ]\n");
+        return exit_FAILED;
+    }
+
+    fname = argv[1];
+    f = fopen(fname, "w");
     if (f == NULL) {
         fprintf(stderr, "genarch.c: can't open %s for writing\n", fname);
         return exit_FAILED;
     }
+
+    if (argc == 3) {
+        return copy_existing_file(f, argv[2]);
+    }
+
     fprintf(f, "/* Parameters derived from machine and compiler architecture. */\n");
     fprintf(f, "/* This file is generated mechanically by genarch.c. */\n");
 
