@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Unix-specific routines for Ghostscript */
 
 #include "pipe_.h"
@@ -107,12 +109,22 @@ gp_do_exit(int exit_status)
 /* ------ Miscellaneous ------ */
 
 /* Get the string corresponding to an OS error number. */
-/* Unix systems support this so inconsistently that we don't attempt */
-/* to figure out whether it's available. */
 const char *
 gp_strerror(int errnum)
 {
+#ifdef HAVE_STRERROR
+    return strerror(errnum);
+#else
     return NULL;
+#endif
+}
+
+/* We don't have a good way to get a serial number here, so just */
+/* return what we always used to: GS_SERIALNUMBER. */
+int
+gp_serialnumber(void)
+{
+    return (int)(gs_serialnumber);
 }
 
 /* read in a MacOS 'resource' from an extended attribute. */
