@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Default device parameters for Ghostscript library */
 #include "memory_.h"		/* for memcpy */
 #include "string_.h"		/* for strlen */
@@ -78,13 +80,14 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
 
     bool seprs = false;
     gs_param_string dns, pcms, profile_array[NUM_DEVICE_PROFILES];
+    gs_param_string proof_profile, link_profile;
     gsicc_rendering_intents_t profile_intents[NUM_DEVICE_PROFILES];
     bool devicegraytok = true;  /* Default if device profile stuct not set */
+    bool usefastcolor = false;  /* set for unmanaged color */
     int k;
     gs_param_float_array msa, ibba, hwra, ma;
     gs_param_string_array scna;
     char null_str[1]={'\0'};
-    gs_param_string temp_str;
 
 #define set_param_array(a, d, s)\
   (a.data = d, a.size = s, a.persistent = false);
@@ -134,8 +137,13 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
             code = gsicc_init_device_profile_struct(dev, NULL, 0);
             code = dev_proc(dev, get_profile)(dev,  &dev_profile);
         } 
+        /* It is possible that the current device profile name is NULL if we 
+           have a pdf14 device in line with a transparency group that is in a 
+           color space specified from a source defined ICC profile. Check for 
+           that here to avoid any access violations.  Bug 692558 */
         for (k = 0; k < NUM_DEVICE_PROFILES; k++) {
-            if (dev_profile->device_profile[k] == NULL) {
+            if (dev_profile->device_profile[k] == NULL 
+                || dev_profile->device_profile[k]->name == NULL) {
                 param_string_from_string(profile_array[k], null_str);
                 profile_intents[k] = gsPERCEPTUAL;
             } else {
@@ -144,15 +152,31 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
                 profile_intents[k] = dev_profile->intent[k];
             }
         }
+        /* The proof and link profile */
+        if (dev_profile->proof_profile == NULL) {
+            param_string_from_string(proof_profile, null_str);
+        } else {
+            param_string_from_string(proof_profile, 
+                                     dev_profile->proof_profile->name);
+        }
+        if (dev_profile->link_profile == NULL) {
+            param_string_from_string(link_profile, null_str);
+        } else {
+            param_string_from_string(link_profile, 
+                                     dev_profile->link_profile->name);
+        }
         devicegraytok = dev_profile->devicegraytok;
+        usefastcolor = dev_profile->usefastcolor;
     } else {
         for (k = 0; k < NUM_DEVICE_PROFILES; k++) {
             param_string_from_string(profile_array[k], null_str);
             profile_intents[k] = gsPERCEPTUAL;
         }
+        param_string_from_string(proof_profile, null_str);
+        param_string_from_string(link_profile, null_str);
     }
     /* Transmit the values. */
-       if (
+    if (
         /* Standard parameters */
         (code = param_write_name(plist, "OutputDevice", &dns)) < 0 ||
 #ifdef PAGESIZE_IS_MEDIASIZE
@@ -178,14 +202,17 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
         /* Note:  if change is made in NUM_DEVICE_PROFILES we need to name
            that profile here for the device parameter on the command line */
         (code = param_write_bool(plist, "DeviceGrayToK", &devicegraytok)) < 0 ||
+        (code = param_write_bool(plist, "UseFastColor", &usefastcolor)) < 0 ||
         (code = param_write_string(plist,"OutputICCProfile", &(profile_array[0]))) < 0 ||
         (code = param_write_string(plist,"GraphicICCProfile", &(profile_array[1]))) < 0 ||
         (code = param_write_string(plist,"ImageICCProfile", &(profile_array[2]))) < 0 ||
         (code = param_write_string(plist,"TextICCProfile", &(profile_array[3]))) < 0 ||
-        (code = param_write_int(plist,"RenderIntent", &(profile_intents[0]))) < 0 ||
-        (code = param_write_int(plist,"GraphicIntent", &(profile_intents[1]))) < 0 ||
-        (code = param_write_int(plist,"ImageIntent", &(profile_intents[2]))) < 0 ||
-        (code = param_write_int(plist,"TextIntent", &(profile_intents[3]))) < 0 ||
+        (code = param_write_string(plist,"ProofProfile", &(proof_profile))) < 0 ||
+        (code = param_write_string(plist,"DeviceLinkProfile", &(link_profile))) < 0 ||
+        (code = param_write_int(plist,"RenderIntent", (const int *) (&(profile_intents[0])))) < 0 ||
+        (code = param_write_int(plist,"GraphicIntent", (const int *) &(profile_intents[1]))) < 0 ||
+        (code = param_write_int(plist,"ImageIntent", (const int *) &(profile_intents[2]))) < 0 ||
+        (code = param_write_int(plist,"TextIntent", (const int *) &(profile_intents[3]))) < 0 ||
         (code = param_write_int_array(plist, "HWSize", &hwsa)) < 0 ||
         (code = param_write_float_array(plist, ".HWMargins", &hwma)) < 0 ||
         (code = param_write_float_array(plist, ".MarginsHWResolution", &mhwra)) < 0 ||
@@ -484,6 +511,36 @@ gx_default_put_graytok(bool graytok, gx_device * dev)
     }
 }
 
+static void
+gx_default_put_usefastcolor(bool fastcolor, gx_device * dev) 
+{
+    int code;
+    cmm_dev_profile_t *profile_struct;
+
+    if (dev->procs.get_profile == NULL) {
+        /* This is an odd case where the device has not yet fully been 
+           set up with its procedures yet.  We want to make sure that
+           we catch this so we assume here that we are dealing with 
+           the target device.  For now allocate the profile structure
+           but do not intialize the profile yet as the color info 
+           may not be fully set up at this time.  */
+        if (dev->icc_struct == NULL) {
+            /* Allocate at this time the structure */
+            dev->icc_struct = gsicc_new_device_profile_array(dev->memory);
+        }
+        dev->icc_struct->usefastcolor = fastcolor;
+    } else {
+        code = dev_proc(dev, get_profile)(dev,  &profile_struct);
+        if (profile_struct == NULL) {
+            /* Create now  */
+            dev->icc_struct = gsicc_new_device_profile_array(dev->memory);
+            profile_struct =  dev->icc_struct;
+        }
+        profile_struct->usefastcolor = fastcolor;
+    }
+}
+
+
 static void
 gx_default_put_intent(gsicc_profile_types_t icc_intent, gx_device * dev,  
                    gsicc_profile_types_t index) 
@@ -576,13 +633,14 @@ gx_default_put_params(gx_device * dev, gs_param_list * plist)
     int leadingedge = dev->LeadingEdge;
     int k;
     bool devicegraytok = true;
-
+    bool usefastcolor = false;
 
     if (dev->icc_struct != NULL) {
         for (k = 0; k < NUM_DEVICE_PROFILES; k++) {
             rend_intent[k] = dev->icc_struct->intent[k];
         }
         devicegraytok = dev->icc_struct->devicegraytok;
+        usefastcolor = dev->icc_struct->usefastcolor;
     } else {
         for (k = 0; k < NUM_DEVICE_PROFILES; k++) {
             rend_intent[k] = gsPERCEPTUAL;
@@ -777,6 +835,12 @@ nce:
     if ((code = param_read_string(plist, "TextICCProfile", &icc_pro)) != 1) {
         gx_default_put_icc(&icc_pro, dev, gsTEXTPROFILE); 
     }
+    if ((code = param_read_string(plist, "ProofProfile", &icc_pro)) != 1) {
+        gx_default_put_icc(&icc_pro, dev, gsPROOFPROFILE); 
+    }    
+    if ((code = param_read_string(plist, "DeviceLinkProfile", &icc_pro)) != 1) {
+        gx_default_put_icc(&icc_pro, dev, gsLINKPROFILE); 
+    }
     if ((code = param_read_int(plist, (param_name = "RenderIntent"), 
                                                     &(rend_intent[0]))) < 0) {
         ecode = code;
@@ -802,6 +866,11 @@ nce:
         ecode = code;
         param_signal_error(plist, param_name, ecode);
     }
+    if ((code = param_read_bool(plist, (param_name = "UseFastColor"), 
+                                                        &usefastcolor)) < 0) {
+        ecode = code;
+        param_signal_error(plist, param_name, ecode);
+    }
     if ((code = param_read_bool(plist, (param_name = "UseCIEColor"), &ucc)) < 0) {
         ecode = code;
         param_signal_error(plist, param_name, ecode);
@@ -1003,6 +1072,7 @@ nce:
         gx_default_put_intent(rend_intent[3], dev, gsTEXTPROFILE); 
     }
     gx_default_put_graytok(devicegraytok, dev);
+    gx_default_put_usefastcolor(usefastcolor, dev);
     return 0;
 }
 
