@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id$ */
+
 /* Command list document- and page-level code. */
 #include "memory_.h"
 #include "string_.h"
@@ -181,7 +183,7 @@ const gx_device_procs gs_clist_device_procs = {
     gx_forward_encode_color,
     gx_forward_decode_color,
     NULL,                       /* pattern_manage */
-    gx_default_fill_rectangle_hl_color,
+    clist_fill_rectangle_hl_color,
     gx_default_include_color_space,
     gx_default_fill_linear_color_scanline,
     clist_fill_linear_color_trapezoid,
@@ -193,9 +195,12 @@ const gx_device_procs gs_clist_device_procs = {
     NULL,                      /* pop_transparency_state */
     NULL,                      /* put_image */
     clist_dev_spec_op,
-    clist_copy_plane,          /* copy plane */
+    clist_copy_planes,         /* copy planes */
     gx_default_get_profile,
-    gx_default_set_graphics_type_tag
+    gx_default_set_graphics_type_tag,
+    clist_strip_copy_rop2,
+    clist_strip_tile_rect_devn,
+    clist_copy_alpha_hl_color,
 };
 
 /*------------------- Choose the implementation -----------------------
@@ -210,6 +215,8 @@ const clist_io_procs_t *clist_io_procs_memory_global = NULL;
 void
 clist_init_io_procs(gx_device_clist *pclist_dev, bool in_memory)
 {
+    /* if clist_io_procs_file_global is NULL, then BAND_LIST_STORAGE=memory */
+    /* was specified in the build, and "file" is not available */
     if (in_memory || clist_io_procs_file_global == NULL)
         pclist_dev->common.page_info.io_procs = clist_io_procs_memory_global;
     else
@@ -401,7 +408,9 @@ clist_init_data(gx_device * dev, byte * init_data, uint data_size)
     cdev->graphics_type_tag = target->graphics_type_tag;	/* initialize to same as target */
 
     /* Call create_buf_device to get the memory planarity set up. */
-    cdev->buf_procs.create_buf_device(&pbdev, target, 0, NULL, NULL, clist_get_band_complexity(0, 0));
+    code = cdev->buf_procs.create_buf_device(&pbdev, target, 0, NULL, NULL, clist_get_band_complexity(0, 0));
+    if (code < 0)
+        return code;
     /* HACK - if the buffer device can't do copy_alpha, disallow */
     /* copy_alpha in the commmand list device as well. */
     if (dev_proc(pbdev, copy_alpha) == gx_no_copy_alpha)
@@ -613,8 +622,8 @@ clist_reset_page(gx_device_clist_writer *cwdev)
     cwdev->page_bfile_end_pos = 0;
     /* Indicate that the colors_used information hasn't been computed. */
     cwdev->page_info.scan_lines_per_colors_used = 0;
-    memset(cwdev->page_info.band_colors_used, 0,
-           sizeof(cwdev->page_info.band_colors_used));
+    memset(cwdev->page_info.band_color_usage, 0,
+           sizeof(cwdev->page_info.band_color_usage));
 }
 
 /* Open the device's bandfiles */
@@ -819,7 +828,7 @@ clist_end_page(gx_device_clist_writer * cldev)
             code = 0;
     }
     if (code >= 0) {
-        clist_compute_colors_used(cldev);
+        clist_compute_color_usage(cldev);
         ecode |= code;
         cldev->page_bfile_end_pos = cldev->page_info.io_procs->ftell(cldev->page_bfile);
     }
@@ -846,6 +855,20 @@ clist_end_page(gx_device_clist_writer * cldev)
     return 0;
 }
 
+gx_color_usage_bits
+gx_color_index2usage(gx_device *dev, gx_color_index color)
+{
+    gx_color_usage_bits bits = 0;
+    int i;
+
+    for (i = 0; i < dev->color_info.num_components; i++) {
+        if (color & dev->color_info.comp_mask[i])
+            bits |= (1<<i);
+    }
+
+    return bits;
+}
+
 /* Compute the set of used colors in the page_info structure.
  *
  * NB: Area for improvement, move states[band] and page_info to clist
@@ -854,7 +877,7 @@ clist_end_page(gx_device_clist_writer * cldev)
  */
 
 void
-clist_compute_colors_used(gx_device_clist_writer *cldev)
+clist_compute_color_usage(gx_device_clist_writer *cldev)
 {
     int nbands = cldev->nbands;
     int bands_per_colors_used =
@@ -864,15 +887,15 @@ clist_compute_colors_used(gx_device_clist_writer *cldev)
 
     cldev->page_info.scan_lines_per_colors_used =
         cldev->page_band_height * bands_per_colors_used;
-    memset(cldev->page_info.band_colors_used, 0,
-           sizeof(cldev->page_info.band_colors_used));
+    memset(cldev->page_info.band_color_usage, 0,
+           sizeof(cldev->page_info.band_color_usage));
     for (band = 0; band < nbands; ++band) {
         int entry = band / bands_per_colors_used;
 
-        cldev->page_info.band_colors_used[entry].or |=
-            cldev->states[band].colors_used.or;
-        cldev->page_info.band_colors_used[entry].slow_rop |=
-            cldev->states[band].colors_used.slow_rop;
+        cldev->page_info.band_color_usage[entry].or |=
+            cldev->states[band].color_usage.or;
+        cldev->page_info.band_color_usage[entry].slow_rop |=
+            cldev->states[band].color_usage.slow_rop;
 
     }
 }
@@ -999,19 +1022,19 @@ clist_get_band(gx_device * dev, int y, int *band_start)
  * default constructor if from == NULL
  */
 void
-clist_copy_band_complexity(gx_band_complexity_t *this, const gx_band_complexity_t *from)
+clist_copy_band_complexity(gx_band_complexity_t *self, const gx_band_complexity_t *from)
 {
     if (from) {
-        memcpy(this, from, sizeof(gx_band_complexity_t));
+        memcpy(self, from, sizeof(gx_band_complexity_t));
     } else {
         /* default */
-        this->uses_color = false;
-        this->nontrivial_rops = false;
+        self->uses_color = false;
+        self->nontrivial_rops = false;
 #if 0
         /* todo: halftone phase */
 
-        this->x0 = 0;
-        this->y0 = 0;
+        self->x0 = 0;
+        self->y0 = 0;
 #endif
     }
 }
@@ -1040,7 +1063,6 @@ clist_icc_searchtable(gx_device_clist_writer *cdev, int64_t hashcode)
 int
 clist_icc_freetable(clist_icctable_t *icc_table, gs_memory_t *memory)
 {
-
     int number_entries;
     clist_icctable_entry_t *curr_entry, *next_entry;
     int k;
@@ -1051,10 +1073,10 @@ clist_icc_freetable(clist_icctable_t *icc_table, gs_memory_t *memory)
     curr_entry = icc_table->head;
     for (k = 0; k < number_entries; k++) {
         next_entry = curr_entry->next;
-        gs_free_object(memory, curr_entry, "clist_icc_freetable");
+        gs_free_object(icc_table->memory, curr_entry, "clist_icc_freetable");
         curr_entry = next_entry;
     }
-    gs_free_object(memory, icc_table, "clist_icc_freetable");
+    gs_free_object(icc_table->memory, icc_table, "clist_icc_freetable");
     return(0);
 }
 
@@ -1135,6 +1157,7 @@ clist_icc_addentry(gx_device_clist_writer *cdev, int64_t hashcode_in, cmm_profil
     clist_icctable_entry_t *entry, *curr_entry;
     int k;
     int64_t hashcode;
+    gs_memory_t *stable_mem = cdev->memory->stable_memory;
 
     /* If the hash code is not valid then compute it now */
     if (icc_profile->hash_is_valid == false) {
@@ -1146,9 +1169,9 @@ clist_icc_addentry(gx_device_clist_writer *cdev, int64_t hashcode_in, cmm_profil
         hashcode = hashcode_in;
     }
     if ( icc_table == NULL ) {
-        entry = (clist_icctable_entry_t *) gs_alloc_struct(cdev->memory,
-                    clist_icctable_entry_t,
-                    &st_clist_icctable_entry, "clist_icc_addentry");
+        entry = (clist_icctable_entry_t *) gs_alloc_struct(stable_mem,
+                               clist_icctable_entry_t, &st_clist_icctable_entry, 
+                               "clist_icc_addentry");
         if (entry == NULL)
             return gs_rethrow(-1, "insufficient memory to allocate entry in icc table");
         entry->next = NULL;
@@ -1157,22 +1180,19 @@ clist_icc_addentry(gx_device_clist_writer *cdev, int64_t hashcode_in, cmm_profil
         entry->serial_data.file_position = -1;
         entry->icc_profile = icc_profile;
         rc_increment(icc_profile);
-        icc_table = gs_alloc_struct(cdev->memory,
-                clist_icctable_t,
-                &st_clist_icctable, "clist_icc_addentry");
-
+        icc_table = gs_alloc_struct(stable_mem, clist_icctable_t, 
+                                    &st_clist_icctable, "clist_icc_addentry");
         if (icc_table == NULL)
             return gs_rethrow(-1, "insufficient memory to allocate icc table");
         icc_table->tablesize = 1;
         icc_table->head = entry;
         icc_table->final = entry;
-
+        icc_table->memory = stable_mem;
         /* For now, we are just going to put the icc_table itself
             at band_range_max + 1.  The ICC profiles are written
             in the cfile at the current stored file position*/
         cdev->icc_table = icc_table;
     } else {
-
         /* First check if we already have this entry */
         curr_entry = icc_table->head;
         for ( k = 0; k < icc_table->tablesize; k++ ) {
@@ -1180,11 +1200,12 @@ clist_icc_addentry(gx_device_clist_writer *cdev, int64_t hashcode_in, cmm_profil
                 return(0);  /* A hit */
             curr_entry = curr_entry->next;
         }
-
          /* Add a new ICC profile */
-        entry = (clist_icctable_entry_t *) gs_alloc_struct(cdev->memory,
-                    clist_icctable_entry_t,
-                    &st_clist_icctable_entry, "clist_icc_addentry");
+        entry = 
+            (clist_icctable_entry_t *) gs_alloc_struct(icc_table->memory, 
+                                                       clist_icctable_entry_t,
+                                                       &st_clist_icctable_entry, 
+                                                       "clist_icc_addentry");
         if (entry == NULL)
             return gs_rethrow(-1, "insufficient memory to allocate entry in icc table");
         entry->next = NULL;
@@ -1280,7 +1301,7 @@ int clist_data_size(const gx_device_clist *cdev, int select)
 
 /* Get command list data. */
 int
-clist_get_data(const gx_device_clist *cdev, int select, int offset, byte *buf, int length)
+clist_get_data(const gx_device_clist *cdev, int select, int64_t offset, byte *buf, int length)
 {
     const gx_band_page_info_t *pinfo = &cdev->common.page_info;
     clist_file_ptr pfile = (!select ? pinfo->bfile : pinfo->cfile);
@@ -1297,11 +1318,11 @@ clist_get_data(const gx_device_clist *cdev, int select, int offset, byte *buf, i
 
 /* Put command list data. */
 int
-clist_put_data(const gx_device_clist *cdev, int select, int offset, const byte *buf, int length)
+clist_put_data(const gx_device_clist *cdev, int select, int64_t offset, const byte *buf, int length)
 {
     const gx_band_page_info_t *pinfo = &cdev->common.page_info;
     clist_file_ptr pfile = (!select ? pinfo->bfile : pinfo->cfile);
-    int code;
+    int64_t code;
 
     code = pinfo->io_procs->ftell(pfile);
     if (code < 0)
