@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxoprect.c,v 1.2 2010/07/10 22:02:25 Arabidopsis Exp $ */
+
 /* generic (very slow) overprint fill rectangle implementation */
 
 #include "memory_.h"
@@ -24,7 +26,6 @@
 #include "gxoprect.h"
 #include "gsbitops.h"
 
-
 /*
  * Unpack a scanline for a depth < 8. In this case we know the depth is
  * divisor of 8 and thus a power of 2, which implies that 8 / depth is
@@ -167,7 +168,6 @@ pack_scanline_ge8(
     }
 }
 
-
 /*
  * Perform the fill rectangle operation for a non-separable color encoding
  * that requires overprint support. This situation requires that colors be
@@ -188,6 +188,7 @@ int
 gx_overprint_generic_fill_rectangle(
     gx_device *             tdev,
     gx_color_index          drawn_comps,
+    ushort                  k_value,
     int                     x,
     int                     y,
     int                     w,
@@ -259,7 +260,7 @@ gx_overprint_generic_fill_rectangle(
      *
      *  - Data should be returned in the depth of the process color
      *    model. Though this depth could be specified explicitly, there
-     *    is little reason to do so. 
+     *    is little reason to do so.
      *
      *  - Though overprint is much more easily implemented with planar
      *    data, there is no planar version of the copy_color method to
@@ -317,12 +318,24 @@ gx_overprint_generic_fill_rectangle(
             gx_color_index  comps;
             int             j;
             gx_color_value  dest_cvals[GX_DEVICE_COLOR_MAX_COMPONENTS];
-        
+
             if ((code = dev_proc(tdev, decode_color)(tdev, *cp, dest_cvals)) < 0)
                 break;
-            for (j = 0, comps = drawn_comps; comps != 0; ++j, comps >>= 1) {
-                if ((comps & 0x1) != 0)
-                    dest_cvals[j] = src_cvals[j];
+            if (k_value > 0) {
+                /* Have to run through all 3 components */
+                for (j = 0, comps = drawn_comps; j < 3; j++, comps >>= 1) {
+                    if ((comps & 0x1) != 0)
+                        dest_cvals[j] = src_cvals[j];
+                    else {
+                        int temp = (dest_cvals[j] * (256 - k_value));
+                        dest_cvals[j] = temp >> 8;
+                    }
+                }
+            } else {
+                for (j = 0, comps = drawn_comps; comps != 0; ++j, comps >>= 1) {
+                    if ((comps & 0x1) != 0)
+                        dest_cvals[j] = src_cvals[j];
+                }
             }
             *cp = dev_proc(tdev, encode_color)(tdev, dest_cvals);
         }
@@ -345,8 +358,6 @@ gx_overprint_generic_fill_rectangle(
     return code;
 }
 
-
-
 /*
  * Replication of 2 and 4 bit patterns to fill a mem_mono_chunk.
  */
@@ -404,7 +415,6 @@ replicate_color(int depth, mono_fill_chunk color)
     return color;
 }
 
-
 /*
  * Perform the fill rectangle operation for a separable color encoding
  * that requires overprint support.
@@ -525,7 +535,6 @@ gx_overprint_sep_fill_rectangle_1(
     return code;
 }
 
-
 int
 gx_overprint_sep_fill_rectangle_2(
     gx_device *             tdev,
@@ -613,4 +622,3 @@ gx_overprint_sep_fill_rectangle_2(
 
     return code;
 }
-
