@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* PatternType 1 implementation interface */
 /* Requires gxpcolor.h */
 
@@ -25,6 +27,7 @@
  */
 dev_color_proc_fill_rectangle(gx_dc_pattern_fill_rectangle);
 dev_color_proc_fill_rectangle(gx_dc_pure_masked_fill_rect);
+dev_color_proc_fill_rectangle(gx_dc_devn_masked_fill_rect);
 dev_color_proc_fill_rectangle(gx_dc_binary_masked_fill_rect);
 dev_color_proc_fill_rectangle(gx_dc_colored_masked_fill_rect);
 dev_color_proc_fill_rectangle(gx_dc_pat_trans_fill_rectangle);
