@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Private graphics state definition for Ghostscript library */
 
 #ifndef gzstate_INCLUDED
@@ -149,10 +151,9 @@ struct gs_state_s {
     gs_state_client_procs client_procs;
 };
 
-struct_proc_finalize(gs_imager_state_finalize);
 #define public_st_gs_state()	/* in gsstate.c */\
-  gs_public_st_composite_use_final(st_gs_state, gs_state, "gs_state",\
-    gs_state_enum_ptrs, gs_state_reloc_ptrs, gs_imager_state_finalize)
+  gs_public_st_composite(st_gs_state, gs_state, "gs_state",\
+    gs_state_enum_ptrs, gs_state_reloc_ptrs)
 
 /*
  * Enumerate the pointers in a graphics state, other than the ones in the
