@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: sa85d.h,v 1.2 2010/07/10 22:02:23 Arabidopsis Exp $ */
+
 /* ASCII85Decode filter interface */
 /* Requires scommon.h; strimpl.h if any templates are referenced */
 
@@ -24,6 +26,7 @@ typedef struct stream_A85D_state_s {
     int odd;			/* # of odd digits */
     ulong word;			/* word being accumulated */
     bool pdf_rules;             /* hacks and tweaks for PDF */
+    bool require_eod;		/* ~> is required for tokens */
 } stream_A85D_state;
 
 #define private_st_A85D_state()	/* in sfilter2.c */\
@@ -32,7 +35,13 @@ typedef struct stream_A85D_state_s {
 /* We define the initialization procedure here, so that the scanner */
 /* can avoid a procedure call. */
 #define s_A85D_init_inline(ss)\
-  ((ss)->min_left = 1, (ss)->word = 0, (ss)->odd = 0)
+  BEGIN \
+  (ss)->min_left = 1; \
+  (ss)->word = 0; \
+  (ss)->odd = 0; \
+  /* pdf_rules should not be initialized here */ \
+  (ss)->require_eod=false; \
+  END
 extern const stream_template s_A85D_template;
 
 #endif /* sa85d_INCLUDED */
