@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: string_.h,v 1.2 2010/07/10 22:02:20 Arabidopsis Exp $ */
+
 /* Generic substitute for Unix string.h */
 
 #ifndef string__INCLUDED
@@ -29,7 +31,7 @@
 #  endif
 #  include <string.h>
 #  if defined(THINK_C)
-	/* Patch strlen to return a uint rather than a size_t. */
+        /* Patch strlen to return a uint rather than a size_t. */
 #    define strlen (uint)strlen
 #  endif
 #  ifdef MEMORY__NEED_MEMMOVE
