@@ -1,18 +1,23 @@
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
+   All Rights Reserved.
+
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
+*/
+
 /*
     jbig2dec
-
-    Copyright (C) 2001-2005 Artifex Software, Inc.
-
-    This software is distributed under license and may not
-    be copied, modified or distributed except as expressly
-    authorized under the terms of the license contained in
-    the file LICENSE in this distribution.
-
-    For further licensing information refer to http://artifex.com/ or
-    contact Artifex Software, Inc., 7 Mt. Lassen Drive - Suite A-134,
-    San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
+
 /* symbol dictionary segment decode and support */
 
 #ifdef HAVE_CONFIG_H
@@ -43,7 +48,7 @@
 typedef struct {
   bool SDHUFF;
   bool SDREFAGG;
-  int32_t SDNUMINSYMS;
+  uint32_t SDNUMINSYMS;
   Jbig2SymbolDict *SDINSYMS;
   uint32_t SDNUMNEWSYMS;
   uint32_t SDNUMEXSYMS;
@@ -96,12 +101,16 @@ jbig2_sd_new(Jbig2Ctx *ctx, int n_symbols)
      new->glyphs = jbig2_new(ctx, Jbig2Image*, n_symbols);
      new->n_symbols = n_symbols;
    } else {
+     jbig2_error(ctx, JBIG2_SEVERITY_FATAL, -1,
+         "unable to allocate new empty symbol dict");
      return NULL;
    }
 
    if (new->glyphs != NULL) {
      memset(new->glyphs, 0, n_symbols*sizeof(Jbig2Image*));
    } else {
+     jbig2_error(ctx, JBIG2_SEVERITY_FATAL, -1,
+         "unable to allocate glyphs for new empty symbol dict");
      jbig2_free(ctx->allocator, new);
      return NULL;
    }
@@ -157,6 +166,13 @@ jbig2_sd_list_referred(Jbig2Ctx *ctx, Jbig2Segment *segment)
     int dindex = 0;
 
     dicts = jbig2_new(ctx, Jbig2SymbolDict*, n_dicts);
+    if (dicts == NULL)
+    {
+        jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
+            "failed to allocate referred list of symbol dictionaries");
+        return NULL;
+    }
+
     for (index = 0; index < segment->referred_to_segment_count; index++) {
         rsegment = jbig2_find_segment(ctx, segment->referred_to_segments[index]);
         if (rsegment && ((rsegment->flags & 63) == 0)) {
@@ -195,6 +211,9 @@ jbig2_sd_cat(Jbig2Ctx *ctx, int n_dicts, Jbig2SymbolDict **dicts)
     for (i = 0; i < n_dicts; i++)
       for (j = 0; j < dicts[i]->n_symbols; j++)
         new->glyphs[k++] = jbig2_image_clone(ctx, dicts[i]->glyphs[j]);
+  } else {
+    jbig2_error(ctx, JBIG2_SEVERITY_WARNING, -1,
+        "failed to allocate new symbol dictionary");
   }
 
   return new;
@@ -212,17 +231,18 @@ jbig2_decode_symbol_dict(Jbig2Ctx *ctx,
 			 Jbig2ArithCx *GB_stats,
 			 Jbig2ArithCx *GR_stats)
 {
-  Jbig2SymbolDict *SDNEWSYMS;
-  Jbig2SymbolDict *SDEXSYMS;
-  int32_t HCHEIGHT;
+  Jbig2SymbolDict *SDNEWSYMS = NULL;
+  Jbig2SymbolDict *SDEXSYMS = NULL;
+  uint32_t HCHEIGHT;
   uint32_t NSYMSDECODED;
-  int32_t SYMWIDTH, TOTWIDTH;
+  uint32_t SYMWIDTH, TOTWIDTH;
   uint32_t HCFIRSTSYM;
   uint32_t *SDNEWSYMWIDTHS = NULL;
   int SBSYMCODELEN = 0;
   Jbig2WordStream *ws = NULL;
   Jbig2HuffmanState *hs = NULL;
   Jbig2HuffmanTable *SDHUFFRDX = NULL;
+  Jbig2HuffmanTable *SBHUFFRSIZE = NULL;
   Jbig2ArithState *as = NULL;
   Jbig2ArithIntCtx *IADH = NULL;
   Jbig2ArithIntCtx *IADW = NULL;
@@ -232,7 +252,7 @@ jbig2_decode_symbol_dict(Jbig2Ctx *ctx,
   Jbig2ArithIntCtx *IARDX = NULL;
   Jbig2ArithIntCtx *IARDY = NULL;
   int code = 0;
-  Jbig2SymbolDict **refagg_dicts;
+  Jbig2SymbolDict **refagg_dicts = NULL;
   int n_refagg_dicts = 1;
 
   Jbig2TextRegionParams *tparams = NULL;
@@ -242,37 +262,76 @@ jbig2_decode_symbol_dict(Jbig2Ctx *ctx,
   NSYMSDECODED = 0;
 
   ws = jbig2_word_stream_buf_new(ctx, data, size);
+  if (ws == NULL)
+  {
+      jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number,
+          "failed to allocate ws in jbig2_decode_symbol_dict");
+      return NULL;
+  }
+
+  as = jbig2_arith_new(ctx, ws);
+  if (as == NULL)
+  {
+      jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number,
+          "failed to allocate as in jbig2_decode_symbol_dict");
+      jbig2_word_stream_buf_free(ctx, ws);
+      return NULL;
+  }
 
   if (!params->SDHUFF) {
-      as = jbig2_arith_new(ctx, ws);
       IADH = jbig2_arith_int_ctx_new(ctx);
       IADW = jbig2_arith_int_ctx_new(ctx);
       IAEX = jbig2_arith_int_ctx_new(ctx);
       IAAI = jbig2_arith_int_ctx_new(ctx);
+      if ((IADH == NULL) || (IADW == NULL) || (IAEX == NULL) || (IAAI == NULL))
+      {
+          jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number,
+              "failed to allocate storage for symbol bitmap");
+          goto cleanup1;
+      }
       if (params->SDREFAGG) {
-	  int tmp = params->SDINSYMS->n_symbols + params->SDNUMNEWSYMS;
-	  for (SBSYMCODELEN = 0; (1 << SBSYMCODELEN) < tmp; SBSYMCODELEN++);
-	  IAID = jbig2_arith_iaid_ctx_new(ctx, SBSYMCODELEN);
-	  IARDX = jbig2_arith_int_ctx_new(ctx);
-	  IARDY = jbig2_arith_int_ctx_new(ctx);
+          int64_t tmp = params->SDNUMINSYMS + params->SDNUMNEWSYMS;
+          for (SBSYMCODELEN = 0; (1 << SBSYMCODELEN) < tmp; SBSYMCODELEN++);
+          IAID = jbig2_arith_iaid_ctx_new(ctx, SBSYMCODELEN);
+          IARDX = jbig2_arith_int_ctx_new(ctx);
+          IARDY = jbig2_arith_int_ctx_new(ctx);
+          if ((IAID == NULL) || (IARDX == NULL) || (IARDY == NULL))
+          {
+              jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number,
+                  "failed to allocate storage for symbol bitmap");
+              goto cleanup2;
+          }
       }
   } else {
       jbig2_error(ctx, JBIG2_SEVERITY_DEBUG, segment->number,
-	"huffman coded symbol dictionary");
+          "huffman coded symbol dictionary");
       hs = jbig2_huffman_new(ctx, ws);
-      SDHUFFRDX = jbig2_build_huffman_table(ctx,
-				&jbig2_huffman_params_O);
+      SDHUFFRDX = jbig2_build_huffman_table(ctx, &jbig2_huffman_params_O);
+      SBHUFFRSIZE = jbig2_build_huffman_table(ctx, &jbig2_huffman_params_A);
+      if ( (hs == NULL) || (SDHUFFRDX == NULL) || (SBHUFFRSIZE == NULL))
+      {
+          jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number,
+              "failed to allocate storage for symbol bitmap");
+          goto cleanup2;
+      }
       if (!params->SDREFAGG) {
 	  SDNEWSYMWIDTHS = jbig2_new(ctx, uint32_t, params->SDNUMNEWSYMS);
 	  if (SDNEWSYMWIDTHS == NULL) {
 	    jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
-		"could not allocate storage for symbol widths");
-	    return NULL;
+              "could not allocate storage for (%u) symbol widths",
+              params->SDNUMNEWSYMS);
+	    goto cleanup2;
 	  }
       }
   }
 
   SDNEWSYMS = jbig2_sd_new(ctx, params->SDNUMNEWSYMS);
+  if (SDNEWSYMS == NULL) {
+      jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number,
+          "could not allocate storage for (%u) new symbols",
+          params->SDNUMNEWSYMS);
+      goto cleanup2;
+  }
 
   /* 6.5.5 (4a) */
   while (NSYMSDECODED < params->SDNUMNEWSYMS) {
@@ -297,10 +356,9 @@ jbig2_decode_symbol_dict(Jbig2Ctx *ctx,
       HCFIRSTSYM = NSYMSDECODED;
 
       if (HCHEIGHT < 0) {
-	  /* todo: mem cleanup */
-	  code = jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
-			   "Invalid HCHEIGHT value");
-          return NULL;
+          code = jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
+              "Invalid HCHEIGHT value");
+          goto cleanup2;
       }
 #ifdef JBIG2_DEBUG
       jbig2_error(ctx, JBIG2_SEVERITY_DEBUG, segment->number,
@@ -310,20 +368,13 @@ jbig2_decode_symbol_dict(Jbig2Ctx *ctx,
         "decoding height class %d with %d syms decoded", HCHEIGHT, NSYMSDECODED);
 
       for (;;) {
-	  /* check for broken symbol table */
- 	  if (NSYMSDECODED > params->SDNUMNEWSYMS)
-	    {
-	      /* todo: mem cleanup? */
-	      jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number,
-	        "No OOB signalling end of height class %d", HCHEIGHT);
-	      break;
-	    }
 	  /* 6.5.7 */
 	  if (params->SDHUFF) {
 	      DW = jbig2_huffman_get(hs, params->SDHUFFDW, &code);
 	  } else {
 	      code = jbig2_arith_int_decode(IADW, as, &DW);
 	  }
+          if (code < 0) goto cleanup4;
 
 	  /* 6.5.5 (4c.i) */
 	  if (code == 1) {
@@ -331,14 +382,22 @@ jbig2_decode_symbol_dict(Jbig2Ctx *ctx,
 	    " OOB signals end of height class %d", HCHEIGHT);
 	    break;
 	  }
+
+	  /* check for broken symbol table */
+          if (NSYMSDECODED >= params->SDNUMNEWSYMS)
+          {
+              jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
+                  "No OOB signalling end of height class %d", HCHEIGHT);
+              goto cleanup4;
+          }
+
 	  SYMWIDTH = SYMWIDTH + DW;
 	  TOTWIDTH = TOTWIDTH + SYMWIDTH;
 	  if (SYMWIDTH < 0) {
-	      /* todo: mem cleanup */
-              code = jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
-                "Invalid SYMWIDTH value (%d) at symbol %d", SYMWIDTH, NSYMSDECODED+1);
-              return NULL;
-          }
+          code = jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
+              "Invalid SYMWIDTH value (%d) at symbol %d", SYMWIDTH, NSYMSDECODED+1);
+          goto cleanup4;
+      }
 #ifdef JBIG2_DEBUG
 	  jbig2_error(ctx, JBIG2_SEVERITY_DEBUG, segment->number,
             "SYMWIDTH = %d TOTWIDTH = %d", SYMWIDTH, TOTWIDTH);
@@ -364,16 +423,21 @@ jbig2_decode_symbol_dict(Jbig2Ctx *ctx,
 		  memcpy(region_params.gbat, params->sdat, sdat_bytes);
 
 		  image = jbig2_image_new(ctx, SYMWIDTH, HCHEIGHT);
+          if (image == NULL)
+          {
+              code = jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
+                  "failed to allocate image in jbig2_decode_symbol_dict");
+              goto cleanup4;
+          }
 
 		  code = jbig2_decode_generic_region(ctx, segment, &region_params,
-						     as, image, GB_stats);
-		  /* todo: handle errors */
-
-                  SDNEWSYMS->glyphs[NSYMSDECODED] = image;
+              as, image, GB_stats);
+          if (code < 0) goto cleanup4;
 
+          SDNEWSYMS->glyphs[NSYMSDECODED] = image;
 	      } else {
-                  /* 6.5.8.2 refinement/aggregate symbol */
-                  uint32_t REFAGGNINST;
+          /* 6.5.8.2 refinement/aggregate symbol */
+          uint32_t REFAGGNINST;
 
 		  if (params->SDHUFF) {
 		      REFAGGNINST = jbig2_huffman_get(hs, params->SDHUFFAGGINST, &code);
@@ -382,8 +446,8 @@ jbig2_decode_symbol_dict(Jbig2Ctx *ctx,
 		  }
 		  if (code || (int32_t)REFAGGNINST <= 0) {
 		      code = jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
-			"invalid number of symbols or OOB in aggregate glyph");
-		      return NULL;
+                  "invalid number of symbols or OOB in aggregate glyph");
+		      goto cleanup4;
 		  }
 
 		  jbig2_error(ctx, JBIG2_SEVERITY_DEBUG, segment->number,
@@ -402,16 +466,15 @@ jbig2_decode_symbol_dict(Jbig2Ctx *ctx,
 		          if (refagg_dicts == NULL) {
 			      code = jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
 			       "Out of memory allocating dictionary array");
-		              return NULL;
+		              goto cleanup4;
 		          }
 		          refagg_dicts[0] = jbig2_sd_new(ctx, params->SDNUMINSYMS + params->SDNUMNEWSYMS);
 		          if (refagg_dicts[0] == NULL) {
 			      code = jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
 			       "Out of memory allocating symbol dictionary");
 		              jbig2_free(ctx->allocator, refagg_dicts);
-		              return NULL;
+		              goto cleanup4;
 		          }
-		          refagg_dicts[0]->n_symbols = params->SDNUMINSYMS + params->SDNUMNEWSYMS;
 		          for (i=0;i < params->SDNUMINSYMS;i++)
 		          {
 			      refagg_dicts[0]->glyphs[i] = jbig2_image_clone(ctx, params->SDINSYMS->glyphs[i]);
@@ -421,11 +484,9 @@ jbig2_decode_symbol_dict(Jbig2Ctx *ctx,
 			  if (tparams == NULL) {
 			      code = jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
 			      "Out of memory creating text region params");
-			      jbig2_sd_release(ctx, refagg_dicts[0]);
-			      jbig2_free(ctx->allocator, refagg_dicts);
-			      return NULL;
+			      goto cleanup4;
 			  }
-    		          if (!params->SDHUFF) {
+              if (!params->SDHUFF) {
 			      /* Values from Table 17, section 6.5.8.2 (2) */
 			      tparams->IADT = jbig2_arith_int_ctx_new(ctx);
 			      tparams->IAFS = jbig2_arith_int_ctx_new(ctx);
@@ -468,19 +529,16 @@ jbig2_decode_symbol_dict(Jbig2Ctx *ctx,
 		      }
 		      tparams->SBNUMINSTANCES = REFAGGNINST;
 
-      		      image = jbig2_image_new(ctx, SYMWIDTH, HCHEIGHT);
+              image = jbig2_image_new(ctx, SYMWIDTH, HCHEIGHT);
 		      if (image == NULL) {
-			  code = jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
-			   "Out of memory creating symbol image");
-		          jbig2_free(ctx->allocator, tparams);
-			  jbig2_sd_release(ctx, refagg_dicts[0]);
-		          jbig2_free(ctx->allocator, refagg_dicts);
-		          return NULL;
+                  code = jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
+                      "Out of memory creating symbol image");
+                  goto cleanup4;
 		      }
 
 		      /* multiple symbols are handled as a text region */
 		      jbig2_decode_text_region(ctx, segment, tparams, (const Jbig2SymbolDict * const *)refagg_dicts,
-			  n_refagg_dicts, image, data, size, GR_stats, as, as ? NULL : ws);
+			  n_refagg_dicts, image, data, size, GR_stats, as, ws);
 
 		      SDNEWSYMS->glyphs[NSYMSDECODED] = image;
 		      refagg_dicts[0]->glyphs[params->SDNUMINSYMS + NSYMSDECODED] = jbig2_image_clone(ctx, SDNEWSYMS->glyphs[NSYMSDECODED]);
@@ -491,29 +549,48 @@ jbig2_decode_symbol_dict(Jbig2Ctx *ctx,
 		      Jbig2Image *image;
 		      uint32_t ID;
 		      int32_t RDX, RDY;
-		      int ninsyms = params->SDINSYMS->n_symbols;
+		      int BMSIZE = 0;
+		      int ninsyms = params->SDNUMINSYMS;
+		      int code1 = 0;
+		      int code2 = 0;
+		      int code3 = 0;
 
+		      /* 6.5.8.2.2 (2, 3, 4, 5) */
 		      if (params->SDHUFF) {
-			  ID = jbig2_huffman_get_bits(hs, SBSYMCODELEN);
-			  RDX = jbig2_huffman_get(hs, SDHUFFRDX, &code);
-			  RDY = jbig2_huffman_get(hs, SDHUFFRDX, &code);
+		          ID = jbig2_huffman_get_bits(hs, SBSYMCODELEN);
+		          RDX = jbig2_huffman_get(hs, SDHUFFRDX, &code1);
+		          RDY = jbig2_huffman_get(hs, SDHUFFRDX, &code2);
+		          BMSIZE = jbig2_huffman_get(hs, SBHUFFRSIZE, &code3);
+		          jbig2_huffman_skip(hs);
 		      } else {
-			  code = jbig2_arith_iaid_decode(IAID, as, (int32_t*)&ID);
-		          code = jbig2_arith_int_decode(IARDX, as, &RDX);
-		          code = jbig2_arith_int_decode(IARDY, as, &RDY);
+		          code1 = jbig2_arith_iaid_decode(IAID, as, (int32_t*)&ID);
+		          code2 = jbig2_arith_int_decode(IARDX, as, &RDX);
+		          code3 = jbig2_arith_int_decode(IARDY, as, &RDY);
+		      }
+
+		      if ((code1 < 0) || (code2 < 0) || (code3 < 0))
+		      {
+		          code = jbig2_error(ctx, JBIG2_SEVERITY_FATAL,
+		              segment->number, "failed to decode data");
+		          goto cleanup4;
 		      }
 
 		      if (ID >= ninsyms+NSYMSDECODED) {
-			code = jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
-			  "refinement references unknown symbol %d", ID);
-			return NULL;
+                  code = jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
+                      "refinement references unknown symbol %d", ID);
+                  goto cleanup4;
 		      }
 
 		      jbig2_error(ctx, JBIG2_SEVERITY_DEBUG, segment->number,
-			"symbol is a refinement of id %d with the refinement applied at (%d,%d)",
-			ID, RDX, RDY);
+                  "symbol is a refinement of id %d with the "
+                  "refinement applied at (%d,%d)", ID, RDX, RDY);
 
 		      image = jbig2_image_new(ctx, SYMWIDTH, HCHEIGHT);
+              if (image == NULL) {
+                  code = jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
+                      "Out of memory creating symbol image");
+                  goto cleanup4;
+              }
 
 		      /* Table 18 */
 		      rparams.GRTEMPLATE = params->SDRTEMPLATE;
@@ -529,6 +606,11 @@ jbig2_decode_symbol_dict(Jbig2Ctx *ctx,
 
 		      SDNEWSYMS->glyphs[NSYMSDECODED] = image;
 
+		      /* 6.5.8.2.2 (7) */
+		      if (params->SDHUFF) {
+		          if (BMSIZE == 0) BMSIZE = image->height * image->stride;
+		          jbig2_huffman_advance(hs, BMSIZE);
+		      }
 		  }
                }
 
@@ -553,14 +635,14 @@ jbig2_decode_symbol_dict(Jbig2Ctx *ctx,
 	    SDNEWSYMWIDTHS[NSYMSDECODED] = SYMWIDTH;
 	  }
 
+          /* 6.5.5 (4c.iv) */
+          NSYMSDECODED = NSYMSDECODED + 1;
+
 	  jbig2_error(ctx, JBIG2_SEVERITY_DEBUG, segment->number,
-            "decoded symbol %d of %d (%dx%d)",
+            "decoded symbol %u of %u (%ux%u)",
 		NSYMSDECODED, params->SDNUMNEWSYMS,
 		SYMWIDTH, HCHEIGHT);
 
-	  /* 6.5.5 (4c.iv) */
-	  NSYMSDECODED = NSYMSDECODED + 1;
-
       } /* end height class decode loop */
 
       /* 6.5.5 (4d) */
@@ -573,8 +655,7 @@ jbig2_decode_symbol_dict(Jbig2Ctx *ctx,
 	if (code || (BMSIZE < 0)) {
 	  jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
 	    "error decoding size of collective bitmap!");
-	  /* todo: memory cleanup */
-	  return NULL;
+	  goto cleanup4;
 	}
 
 	/* skip any bits before the next byte boundary */
@@ -584,8 +665,7 @@ jbig2_decode_symbol_dict(Jbig2Ctx *ctx,
 	if (image == NULL) {
 	  jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
 	    "could not allocate collective bitmap image!");
-	  /* todo: memory cleanup */
-	  return NULL;
+	  goto cleanup4;
 	}
 
 	if (BMSIZE == 0) {
@@ -619,8 +699,8 @@ jbig2_decode_symbol_dict(Jbig2Ctx *ctx,
 	  if (code) {
 	    jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
 	      "error decoding MMR bitmap image!");
-	    /* todo: memory cleanup */
-	    return NULL;
+	    jbig2_image_release(ctx, image);
+	    goto cleanup4;
 	  }
 	}
 
@@ -632,8 +712,14 @@ jbig2_decode_symbol_dict(Jbig2Ctx *ctx,
 	for (j = HCFIRSTSYM; j < NSYMSDECODED; j++) {
 	  Jbig2Image *glyph;
 	  glyph = jbig2_image_new(ctx, SDNEWSYMWIDTHS[j], HCHEIGHT);
-	  jbig2_image_compose(ctx, glyph, image,
-		-x, 0, JBIG2_COMPOSE_REPLACE);
+      if (glyph == NULL)
+      {
+          jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
+              "failed to copy the collective bitmap into symbol dictionary");
+          jbig2_image_release(ctx, image);
+          goto cleanup4;
+      }
+	  jbig2_image_compose(ctx, glyph, image, -x, 0, JBIG2_COMPOSE_REPLACE);
 	  x += SDNEWSYMWIDTHS[j];
 	  SDNEWSYMS->glyphs[j] = glyph;
 	}
@@ -642,6 +728,54 @@ jbig2_decode_symbol_dict(Jbig2Ctx *ctx,
 
   } /* end of symbol decode loop */
 
+  /* 6.5.10 */
+  SDEXSYMS = jbig2_sd_new(ctx, params->SDNUMEXSYMS);
+  if (SDEXSYMS == NULL)
+  {
+      jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
+          "failed to allocate symbols exported from symbols dictionary");
+      goto cleanup4;
+  }
+  else
+  {
+    int i = 0;
+    int j = 0;
+    int k;
+    int exflag = 0;
+    int64_t limit = params->SDNUMINSYMS + params->SDNUMNEWSYMS;
+    int32_t exrunlength;
+
+    while (i < limit) {
+      if (params->SDHUFF)
+        exrunlength = jbig2_huffman_get(hs, SBHUFFRSIZE, &code);
+      else
+        code = jbig2_arith_int_decode(IAEX, as, &exrunlength);
+      if (code || (exrunlength > limit - i)) {
+        if (code)
+          jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
+            "failed to decode exrunlength for exported symbols");
+        else
+          jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
+            "runlength too large in export symbol table (%d > %d - %d)\n",
+            exrunlength, params->SDNUMEXSYMS, j);
+        /* skip to the cleanup code and return SDEXSYMS = NULL */
+        jbig2_sd_release(ctx, SDEXSYMS);
+        SDEXSYMS = NULL;
+        break;
+      }
+      for(k = 0; k < exrunlength; k++) {
+        if (exflag) {
+          SDEXSYMS->glyphs[j++] = (i < params->SDNUMINSYMS) ?
+            jbig2_image_clone(ctx, params->SDINSYMS->glyphs[i]) :
+            jbig2_image_clone(ctx, SDNEWSYMS->glyphs[i-params->SDNUMINSYMS]);
+        }
+        i++;
+      }
+      exflag = !exflag;
+    }
+  }
+
+cleanup4:
   if (tparams != NULL)
   {
       if (!params->SDHUFF)
@@ -668,74 +802,33 @@ jbig2_decode_symbol_dict(Jbig2Ctx *ctx,
           jbig2_release_huffman_table(ctx, tparams->SBHUFFRDH);
       }
       jbig2_free(ctx->allocator, tparams);
-      tparams = NULL;
+  }
+  if (refagg_dicts != NULL)
+  {
       jbig2_sd_release(ctx, refagg_dicts[0]);
       jbig2_free(ctx->allocator, refagg_dicts);
   }
 
-  jbig2_free(ctx->allocator, GB_stats);
-
-  /* 6.5.10 */
-  SDEXSYMS = jbig2_sd_new(ctx, params->SDNUMEXSYMS);
-  {
-    int i = 0;
-    int j = 0;
-    int k, m, exflag = 0;
-    int32_t exrunlength;
-
-    if (params->SDINSYMS != NULL)
-      m = params->SDINSYMS->n_symbols;
-    else
-      m = 0;
-    while (j < params->SDNUMEXSYMS) {
-      if (params->SDHUFF)
-      	/* FIXME: implement reading from huff table B.1 */
-        exrunlength = exflag ? params->SDNUMEXSYMS : 0;
-      else
-        code = jbig2_arith_int_decode(IAEX, as, &exrunlength);
-      if (exflag && exrunlength > params->SDNUMEXSYMS - j) {
-        jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
-          "runlength too large in export symbol table (%d > %d - %d)\n",
-          exrunlength, params->SDNUMEXSYMS, j);
-        jbig2_sd_release(ctx, SDEXSYMS);
-        /* skip to the cleanup code and return SDEXSYMS = NULL */
-        SDEXSYMS = NULL;
-        break;
-      }
-      for(k = 0; k < exrunlength; k++) {
-        if (exflag) {
-          SDEXSYMS->glyphs[j++] = (i < m) ?
-            jbig2_image_clone(ctx, params->SDINSYMS->glyphs[i]) :
-            jbig2_image_clone(ctx, SDNEWSYMS->glyphs[i-m]);
-        }
-        i++;
-      }
-      exflag = !exflag;
-    }
-  }
-
+cleanup2:
   jbig2_sd_release(ctx, SDNEWSYMS);
-
-  if (!params->SDHUFF) {
-      jbig2_arith_int_ctx_free(ctx, IADH);
-      jbig2_arith_int_ctx_free(ctx, IADW);
-      jbig2_arith_int_ctx_free(ctx, IAEX);
-      jbig2_arith_int_ctx_free(ctx, IAAI);
-      if (params->SDREFAGG) {
-        jbig2_arith_iaid_ctx_free(ctx, IAID);
-        jbig2_arith_int_ctx_free(ctx, IARDX);
-        jbig2_arith_int_ctx_free(ctx, IARDY);
-      }
-      jbig2_free(ctx->allocator, as);
-  } else {
-      if (params->SDREFAGG) {
-	jbig2_free(ctx->allocator, SDNEWSYMWIDTHS);
-      }
-      jbig2_release_huffman_table(ctx, SDHUFFRDX);
-      jbig2_free(ctx->allocator, hs);
+  if (params->SDHUFF && !params->SDREFAGG)
+  {
+      jbig2_free(ctx->allocator, SDNEWSYMWIDTHS);
   }
+  jbig2_release_huffman_table(ctx, SDHUFFRDX);
+  jbig2_release_huffman_table(ctx, SBHUFFRSIZE);
+  jbig2_huffman_free(ctx, hs);
+  jbig2_arith_iaid_ctx_free(ctx, IAID);
+  jbig2_arith_int_ctx_free(ctx, IARDX);
+  jbig2_arith_int_ctx_free(ctx, IARDY);
 
+cleanup1:
   jbig2_word_stream_buf_free(ctx, ws);
+  jbig2_free(ctx->allocator, as);
+  jbig2_arith_int_ctx_free(ctx, IADH);
+  jbig2_arith_int_ctx_free(ctx, IADW);
+  jbig2_arith_int_ctx_free(ctx, IAEX);
+  jbig2_arith_int_ctx_free(ctx, IAAI);
 
   return SDEXSYMS;
 }
@@ -759,25 +852,22 @@ jbig2_symbol_dictionary(Jbig2Ctx *ctx, Jbig2Segment *segment,
 
   /* 7.4.2.1.1 */
   flags = jbig2_get_uint16(segment_data);
+
+  /* zero params to ease cleanup later */
+  memset(&params, 0, sizeof(Jbig2SymbolDictParams));
+
   params.SDHUFF = flags & 1;
   params.SDREFAGG = (flags >> 1) & 1;
   params.SDTEMPLATE = (flags >> 10) & 3;
   params.SDRTEMPLATE = (flags >> 12) & 1;
 
-  params.SDHUFFDH = NULL;
-  params.SDHUFFDW = NULL;
-  params.SDHUFFBMSIZE = NULL;
-  params.SDHUFFAGGINST = NULL;
-
   if (params.SDHUFF) {
     switch ((flags & 0x000c) >> 2) {
       case 0: /* Table B.4 */
-	params.SDHUFFDH = jbig2_build_huffman_table(ctx,
-		                       &jbig2_huffman_params_D);
+	params.SDHUFFDH = jbig2_build_huffman_table(ctx, &jbig2_huffman_params_D);
 	break;
       case 1: /* Table B.5 */
-	params.SDHUFFDH = jbig2_build_huffman_table(ctx,
-		                       &jbig2_huffman_params_E);
+	params.SDHUFFDH = jbig2_build_huffman_table(ctx, &jbig2_huffman_params_E);
 	break;
       case 3: /* Custom table from referred segment */
         huffman_params = jbig2_find_table(ctx, segment, table_index);
@@ -788,28 +878,32 @@ jbig2_symbol_dictionary(Jbig2Ctx *ctx, Jbig2Segment *segment,
         params.SDHUFFDH = jbig2_build_huffman_table(ctx, huffman_params);
         ++table_index;
         break;
-        /* FIXME: this function leaks memory when error happens.
-           i.e. not calling jbig2_release_huffman_table() */
       case 2:
       default:
 	return jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
 	    "symbol dictionary specified invalid huffman table");
 	break;
     }
+    if (params.SDHUFFDH == NULL)
+    {
+        jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
+            "failed to allocate DH huffman table");
+        goto cleanup;
+    }
+
     switch ((flags & 0x0030) >> 4) {
       case 0: /* Table B.2 */
-	params.SDHUFFDW = jbig2_build_huffman_table(ctx,
-		                       &jbig2_huffman_params_B);
+	params.SDHUFFDW = jbig2_build_huffman_table(ctx, &jbig2_huffman_params_B);
 	break;
       case 1: /* Table B.3 */
-	params.SDHUFFDW = jbig2_build_huffman_table(ctx,
-		                       &jbig2_huffman_params_C);
+	params.SDHUFFDW = jbig2_build_huffman_table(ctx, &jbig2_huffman_params_C);
 	break;
       case 3: /* Custom table from referred segment */
         huffman_params = jbig2_find_table(ctx, segment, table_index);
         if (huffman_params == NULL) {
-            return jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
+            jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
                 "Custom DW huffman table not found (%d)", table_index);
+            break;
         }
         params.SDHUFFDW = jbig2_build_huffman_table(ctx, huffman_params);
         ++table_index;
@@ -820,33 +914,53 @@ jbig2_symbol_dictionary(Jbig2Ctx *ctx, Jbig2Segment *segment,
 	    "symbol dictionary specified invalid huffman table");
 	break;
     }
+    if (params.SDHUFFDW == NULL)
+    {
+        jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
+            "failed to allocate DW huffman table");
+        goto cleanup;
+    }
+
     if (flags & 0x0040) {
         /* Custom table from referred segment */
         huffman_params = jbig2_find_table(ctx, segment, table_index);
         if (huffman_params == NULL) {
-            return jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
+            jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
                 "Custom BMSIZE huffman table not found (%d)", table_index);
+        } else {
+            params.SDHUFFBMSIZE = jbig2_build_huffman_table(ctx, huffman_params);
+            ++table_index;
         }
-        params.SDHUFFBMSIZE = jbig2_build_huffman_table(ctx, huffman_params);
-        ++table_index;
     } else {
 	/* Table B.1 */
-	params.SDHUFFBMSIZE = jbig2_build_huffman_table(ctx,
-					&jbig2_huffman_params_A);
+	params.SDHUFFBMSIZE = jbig2_build_huffman_table(ctx, &jbig2_huffman_params_A);
     }
+    if (params.SDHUFFBMSIZE == NULL)
+    {
+        jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
+            "failed to allocate BMSIZE huffman table");
+        goto cleanup;
+    }
+
     if (flags & 0x0080) {
         /* Custom table from referred segment */
         huffman_params = jbig2_find_table(ctx, segment, table_index);
         if (huffman_params == NULL) {
-            return jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
+            jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
                 "Custom REFAGG huffman table not found (%d)", table_index);
+        } else {
+            params.SDHUFFAGGINST = jbig2_build_huffman_table(ctx, huffman_params);
+            ++table_index;
         }
-        params.SDHUFFAGGINST = jbig2_build_huffman_table(ctx, huffman_params);
-        ++table_index;
     } else {
 	/* Table B.1 */
-	params.SDHUFFAGGINST = jbig2_build_huffman_table(ctx,
-					&jbig2_huffman_params_A);
+	params.SDHUFFAGGINST = jbig2_build_huffman_table(ctx, &jbig2_huffman_params_A);
+    }
+    if (params.SDHUFFAGGINST == NULL)
+    {
+        jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
+            "failed to allocate REFAGG huffman table");
+        goto cleanup;
     }
   }
 
@@ -854,18 +968,21 @@ jbig2_symbol_dictionary(Jbig2Ctx *ctx, Jbig2Segment *segment,
   /* maybe #ifdef CONFORMANCE and a separate routine */
   if (!params.SDHUFF) {
     if (flags & 0x000c) {
-      jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number,
+      jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
 		  "SDHUFF is zero, but contrary to spec SDHUFFDH is not.");
+      goto cleanup;
     }
     if (flags & 0x0030) {
-      jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number,
+      jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
 		  "SDHUFF is zero, but contrary to spec SDHUFFDW is not.");
+      goto cleanup;
     }
   }
 
   if (flags & 0x0080) {
-      jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number,
+      jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
         "bitmap coding context is used (NYI) symbol data likely to be garbage!");
+      goto cleanup;
   }
 
   /* 7.4.2.1.2 */
@@ -879,10 +996,6 @@ jbig2_symbol_dictionary(Jbig2Ctx *ctx, Jbig2Segment *segment,
 	goto too_short;
       memcpy(params.sdrat, segment_data + offset, 4);
       offset += 4;
-  } else {
-      /* sdrat is meaningless if SDRTEMPLATE is 1, but set a value
-         to avoid confusion if anybody looks */
-      memset(params.sdrat, 0, 4);
   }
 
   if (offset + 8 > segment->data_length)
@@ -895,7 +1008,7 @@ jbig2_symbol_dictionary(Jbig2Ctx *ctx, Jbig2Segment *segment,
   offset += 8;
 
   jbig2_error(ctx, JBIG2_SEVERITY_INFO, segment->number,
-	      "symbol dictionary, flags=%04x, %d exported syms, %d new syms",
+	      "symbol dictionary, flags=%04x, %u exported syms, %u new syms",
 	      flags, params.SDNUMEXSYMS, params.SDNUMNEWSYMS);
 
   /* 7.4.2.2 (2) */
@@ -903,35 +1016,58 @@ jbig2_symbol_dictionary(Jbig2Ctx *ctx, Jbig2Segment *segment,
     int n_dicts = jbig2_sd_count_referred(ctx, segment);
     Jbig2SymbolDict **dicts = NULL;
 
-    params.SDINSYMS = NULL;
     if (n_dicts > 0) {
       dicts = jbig2_sd_list_referred(ctx, segment);
+      if (dicts == NULL)
+      {
+          jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number,
+              "failed to allocate dicts in symbol dictionary");
+          goto cleanup;
+      }
       params.SDINSYMS = jbig2_sd_cat(ctx, n_dicts, dicts);
+      if (params.SDINSYMS == NULL)
+      {
+          jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number,
+              "failed to allocate symbol array in symbol dictionary");
+          jbig2_free(ctx->allocator, dicts);
+          goto cleanup;
+      }
+      jbig2_free(ctx->allocator, dicts);
     }
     if (params.SDINSYMS != NULL) {
       params.SDNUMINSYMS = params.SDINSYMS->n_symbols;
-    } else {
-     params.SDNUMINSYMS = 0;
     }
   }
 
-  /* 7.4.2.2 (4) */
-  if (!params.SDHUFF) {
+  /* 7.4.2.2 (3, 4) */
+  if (flags & 0x0100) {
+      jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number,
+        "segment marks bitmap coding context as used (NYI)");
+  } else {
       int stats_size = params.SDTEMPLATE == 0 ? 65536 :
-	params.SDTEMPLATE == 1 ? 8192 : 1024;
+          params.SDTEMPLATE == 1 ? 8192 : 1024;
       GB_stats = jbig2_new(ctx, Jbig2ArithCx, stats_size);
+      if (GB_stats == NULL)
+      {
+          jbig2_error(ctx, JBIG2_SEVERITY_FATAL, -1,
+              "failed to allocate GB_stats in jbig2_symbol_dictionary");
+          goto cleanup;
+      }
       memset(GB_stats, 0, stats_size);
       if (params.SDREFAGG) {
-	stats_size = params.SDRTEMPLATE ? 1 << 10 : 1 << 13;
-	GR_stats = jbig2_new(ctx, Jbig2ArithCx, stats_size);
-	memset(GR_stats, 0, stats_size);
+          stats_size = params.SDRTEMPLATE ? 1 << 10 : 1 << 13;
+          GR_stats = jbig2_new(ctx, Jbig2ArithCx, stats_size);
+          if (GR_stats == NULL)
+          {
+              jbig2_error(ctx, JBIG2_SEVERITY_FATAL, -1,
+                  "failed to allocate GR_stats in jbig2_symbol_dictionary");
+              jbig2_free(ctx->allocator, GB_stats);
+              goto cleanup;
+          }
+          memset(GR_stats, 0, stats_size);
       }
   }
 
-  if (flags & 0x0100) {
-      jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number,
-        "segment marks bitmap coding context as retained (NYI)");
-  }
 
   segment->result = (void *)jbig2_decode_symbol_dict(ctx, segment,
 				  &params,
@@ -942,14 +1078,24 @@ jbig2_symbol_dictionary(Jbig2Ctx *ctx, Jbig2Segment *segment,
   if (segment->result) jbig2_dump_symbol_dict(ctx, segment);
 #endif
 
+  /* 7.4.2.2 (7) */
+  if (flags & 0x0200) {
+      /* todo: retain GB_stats, GR_stats */
+      jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number,
+          "segment marks bitmap coding context as retained (NYI)");
+  } else {
+      jbig2_free(ctx->allocator, GR_stats);
+      jbig2_free(ctx->allocator, GB_stats);
+  }
+
+cleanup:
   if (params.SDHUFF) {
       jbig2_release_huffman_table(ctx, params.SDHUFFDH);
       jbig2_release_huffman_table(ctx, params.SDHUFFDW);
       jbig2_release_huffman_table(ctx, params.SDHUFFBMSIZE);
       jbig2_release_huffman_table(ctx, params.SDHUFFAGGINST);
   }
-
-  /* todo: retain or free GB_stats, GR_stats */
+  jbig2_sd_release(ctx, params.SDINSYMS);
 
   return (segment->result != NULL) ? 0 : -1;
 
