@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Agfa UFST plugin */
 
 /* GS includes : */
@@ -61,9 +63,15 @@
 #include "ipacked.h"	    /* for packed_next */
 #include "iddict.h"
 #include "ifont42.h"        /* for string_array_access_proc */
+#include "gsmalloc.h"
+#include "gsmchunk.h"
 
 #undef frac_bits
 
+#ifndef _MSC_VER
+#define _MSC_VER 0
+#endif
+
 /* UFST includes : */
 #include "cgconfig.h"
 #include "ufstport.h"
@@ -95,6 +103,9 @@
 #endif
 
 #if UFST_VERSION_MAJOR >= 6 && UFST_VERSION_MINOR >= 2
+#undef true
+#undef false
+
 #define false FALSE
 #define true TRUE
 #define UNICODE UFST_UNICODE
@@ -131,8 +142,7 @@ UW16 CGENTRY CGIFFfont (fc);
 
 typedef struct fapi_ufst_server_s fapi_ufst_server;
 
-static gs_memory_t *gs_mem_ctx = NULL;
-static unsigned long long int font_ids;
+static unsigned int font_ids = 0x7fffffff;
 
 #if UFST_REENTRANT
 #define FSA_FROM_SERVER IF_STATE *pIFS = &r->IFS
@@ -173,7 +183,7 @@ struct fapi_ufst_server_s {
     FAPI_server If;
     int bInitialized;
     FAPI_font *ff;
-    i_plugin_client_memory client_mem;
+    gs_memory_t *mem;
     byte *param;
     int param_size;
     IF_STATE IFS;
@@ -201,6 +211,22 @@ static inline fapi_ufst_server *IFS_to_I(IF_STATE *pIFS)
 {   return (fapi_ufst_server *)((char *)pIFS - offset_of(fapi_ufst_server, IFS));
 }
 
+/* assign font_id not based on fchandle value, nor read from font */
+static inline int assign_font_id(void)
+{
+    /* We're only allowed 5 open FCOs at one time
+     * so make sure we don't clash with those
+     */
+     if ((--font_ids) <= (5 << 16)) {
+         /* When we clash with the expected values for FCO's
+          * wrap around. No will ever have enough concurrent
+          * fonts for this to be a problem!
+          */
+         font_ids = 0x7fffffff;
+     }
+     return(font_ids);
+}
+
 /*------------------ FAPI_server members ------------------------------------*/
 
 static inline void release_char_data_inline(fapi_ufst_server *r)
@@ -248,7 +274,7 @@ static FAPI_retcode open_UFST(fapi_ufst_server *r, const byte *server_param, int
     FSA_FROM_SERVER;
 
     if (server_param_size > 0 && server_param) {
-        r->param = (byte *)r->client_mem.alloc(&r->client_mem, server_param_size, "server_params");
+        r->param = (byte *)gs_malloc(r->mem, server_param_size, 1, "server_params");
         if (!r->param) {
             return_error (e_VMerror);
         }
@@ -275,17 +301,17 @@ static FAPI_retcode open_UFST(fapi_ufst_server *r, const byte *server_param, int
             sPlugIn[l] = 0;
             bPlugIn = TRUE;
         } else
-            emprintf(r->client_mem.client_data,
+            emprintf(r->mem,
                      "Warning: Unknown UFST parameter ignored.\n");
     }
 #if !NO_SYMSET_MAPPING
     if (!bSSdir) {
         strcpy(ufst_root_dir, ".");
-        emprintf(r->client_mem.client_data,
+        emprintf(r->mem,
                  "Warning: UFST_SSdir is not specified, will search *.ss files in the curent directory.\n");
     }
 #endif
-    code = gx_UFST_init(r->client_mem.client_data, (const UB8 *)ufst_root_dir);
+    code = gx_UFST_init(r->mem, (const UB8 *)ufst_root_dir);
     if (code < 0)
         return code;
     r->ufst_is_singleton = (code == 1);
@@ -297,7 +323,7 @@ static FAPI_retcode open_UFST(fapi_ufst_server *r, const byte *server_param, int
             return code;
     } else {
 #ifdef FCO_RDR
-        emprintf(r->client_mem.client_data,
+        emprintf(r->mem,
                  "Warning: UFST_PlugIn is not specified, some characters may be missing.\n");
 #endif
     }
@@ -316,7 +342,7 @@ static FAPI_retcode ensure_open(FAPI_server *server, const byte *server_param, i
     {
         code = open_UFST(r, server_param, server_param_size);
         if (code < 0) {
-            emprintf(r->client_mem.client_data, "Error opening the UFST font server.\n");
+            emprintf(r->mem, "Error opening the UFST font server.\n");
             return code;
         }
     }
@@ -440,9 +466,9 @@ static LPUB8 get_TT_glyph(fapi_ufst_server *r, FAPI_font *ff, UW16 chId)
         r->callback_error = e_invalidfont;
         return 0;
     }
-    g = (pcleo_glyph_list_elem *)r->client_mem.alloc(&r->client_mem,
+    g = (pcleo_glyph_list_elem *)gs_malloc(r->mem,
             sizeof(pcleo_glyph_list_elem) +
-            (use_XL_format ? 12 : sizeof(PCLETTO_CHDR)) + glyph_length + 2,
+            (use_XL_format ? 12 : sizeof(PCLETTO_CHDR)) + glyph_length + 2, 1,
             "PCLETTO char");
     if (g == 0) {
         r->callback_error = e_VMerror;
@@ -510,7 +536,7 @@ static LPUB8 get_T1_glyph(fapi_ufst_server *r, FAPI_font *ff, UW16 chId)
         return (NULL);
     }
 
-    cstring = MEMptr(charstring.hdata);
+    cstring = (unsigned char *)MEMptr(charstring.hdata);
 
     if (ff->get_glyph(ff, chId, cstring, glyph_length) != glyph_length)
     {
@@ -537,8 +563,8 @@ static LPUB8 get_T1_glyph(fapi_ufst_server *r, FAPI_font *ff, UW16 chId)
 #else
     ushort glyph_length = ff->get_glyph(ff, chId, 0, 0);
     LPUB8 q;
-    pcleo_glyph_list_elem *g = (pcleo_glyph_list_elem *)r->client_mem.alloc(&r->client_mem,
-                    sizeof(pcleo_glyph_list_elem) + sizeof(PS_CHAR_HDR) + 2 + 2 + glyph_length + 1, "PSEO char");
+    pcleo_glyph_list_elem *g = (pcleo_glyph_list_elem *)gs_malloc(r->mem,
+                    sizeof(pcleo_glyph_list_elem) + sizeof(PS_CHAR_HDR) + 2 + 2 + glyph_length + 1, 1, "PSEO char");
     PS_CHAR_HDR *h;
     ufst_common_font_data *d = (ufst_common_font_data *)r->fc.font_hdr - 1;
     FSA_FROM_SERVER;
@@ -657,12 +683,12 @@ static void pack_pseo_word_array(fapi_ufst_server *r, FAPI_font *ff, UB8 **p,
 static void pack_pseo_fhdr(fapi_ufst_server *r, FAPI_font *ff, UB8 *p)
 {   ushort j, n, skip = 0;
 
-    while ((UL32)p & 0x03) /* align to QUADWORD */
+    while (((uint64_t)p) & 0x03) /* align to QUADWORD */
         PACK_ZERO(p);
     pack_long(&p, 1);  /* format = 1 */
     for (j = 0; j < 6; j++)
         pack_float(&p, ff->get_float(ff, FAPI_FONT_FEATURE_FontMatrix, j));
-    while ((UL32)p & 0x03) /* align to QUADWORD */
+    while (((uint64_t)p) & 0x03) /* align to QUADWORD */
         PACK_ZERO(p);
     /* UFST has no definition for PSEO structure, so implement serialization : */
     PACK_LONG(p, 0, FAPI_FONT_FEATURE_UniqueID);
@@ -727,11 +753,11 @@ static void pack_pseo_fhdr(fapi_ufst_server *r, FAPI_font *ff, UB8 *p)
 }
 
 static char *my_strdup(fapi_ufst_server *r, const char *s, const char *cname)
-{   int l = strlen(s) + 1;
-    char *p = (char *)r->client_mem.alloc(&r->client_mem, l, cname);
+{   int len = strlen(s) + 1;
+    char *p = (char *)gs_malloc(r->mem, len, 1, cname);
 
     if (p != 0)
-        memcpy(p, s, l);
+        memcpy(p, s, len);
     return p;
 }
 
@@ -753,7 +779,7 @@ static FAPI_retcode fco_open(fapi_ufst_server *r, const char *font_file_path, fc
 
         if ((code = CGIFfco_Open(FSA (UB8 *)font_file_path, &fcHandle)) != 0)
             return code;
-        e = (fco_list_elem *)r->client_mem.alloc(&r->client_mem, sizeof(*e), "fco_list_elem");
+        e = (fco_list_elem *)gs_malloc(r->mem, sizeof(*e), 1, "fco_list_elem");
         if (e == 0) {
             CGIFfco_Close(FSA fcHandle);
             return e_VMerror;
@@ -763,7 +789,7 @@ static FAPI_retcode fco_open(fapi_ufst_server *r, const char *font_file_path, fc
         e->file_path = my_strdup(r, font_file_path, "fco_file_path");
         if (e->file_path == 0) {
             CGIFfco_Close(FSA fcHandle);
-            r->client_mem.free(&r->client_mem, e, "fco_list_elem");
+            gs_free(r->mem, e, 0, 0, "fco_list_elem");
             return e_VMerror;
         }
         e->next = r->fco_list;
@@ -822,7 +848,7 @@ static FAPI_retcode make_font_data(fapi_ufst_server *r, const char *font_file_pa
 #endif
         area_length += strlen(font_file_path) + 1;
     }
-    buf = r->client_mem.alloc(&r->client_mem, area_length, "ufst font data");
+    buf = gs_malloc(r->mem, area_length, 1, "ufst font data");
     if (buf == 0)
         return e_VMerror;
 
@@ -843,9 +869,9 @@ static FAPI_retcode make_font_data(fapi_ufst_server *r, const char *font_file_pa
              d->font_id = (e->fcHandle << 16) | ff->subfont;
              d->font_type = FC_FCO_TYPE;
          } else {
-            stream *f = sfopen(font_file_path, "rb", (gs_memory_t *)(r->client_mem.client_data));
+            stream *f = sfopen(font_file_path, "rb", r->mem);
             if (f == NULL) {
-                emprintf1((gs_memory_t *)(r->client_mem.client_data),
+                emprintf1(r->mem,
                           "fapiufst: Can't open %s\n",
                           font_file_path);
                 return e_undefinedfilename;
@@ -864,7 +890,7 @@ static FAPI_retcode make_font_data(fapi_ufst_server *r, const char *font_file_pa
         d->font_type = (ff->is_type1 ? FC_PST1_TYPE : FC_TT_TYPE);
         d->font_id = ff->get_long(ff, FAPI_FONT_FEATURE_UniqueID, 0);
         if (d->font_id < 0) {
-            d->font_id = ++font_ids;
+            d->font_id = assign_font_id();
         }
         h = (PCLETTO_FHDR *)(buf + sizeof(ufst_common_font_data));
         h->fontDescriptorSize = PCLETTOFONTHDRSIZE;
@@ -970,6 +996,7 @@ static FAPI_retcode get_scaled_font(FAPI_server *server, FAPI_font *ff,
     FAPI_retcode code = 0;
     bool use_XL_format = ff->is_mtx_skipped;
     int world_scale = 0;
+    FONT_METRICS fm;
     FSA_FROM_SERVER;
 
     if (ff->is_cid && ff->is_type1 && ff->font_file_path == NULL &&
@@ -989,8 +1016,12 @@ static FAPI_retcode get_scaled_font(FAPI_server *server, FAPI_font *ff,
         prepare_typeface(r, d);
         if (ff->font_file_path != NULL || ff->is_type1) /* such fonts don't use RAW_GLYPH */
             choose_decoding(r, d, xlatmap);
-    } else
+    }
+    else {
         prepare_typeface(r, d);
+        if (ff->font_file_path != NULL || ff->is_type1) /* such fonts don't use RAW_GLYPH */
+            choose_decoding(r, d, xlatmap);
+    }
 
     r->tran_xx = font_scale->matrix[0] / scale, r->tran_xy = font_scale->matrix[1] / scale;
     r->tran_yx = font_scale->matrix[2] / scale, r->tran_yy = font_scale->matrix[3] / scale;
@@ -1053,7 +1084,11 @@ static FAPI_retcode get_scaled_font(FAPI_server *server, FAPI_font *ff,
     if (r->callback_error != 0)
         return r->callback_error;
 
-   return code;
+    code = CGIFfont_metrics(&fm);
+    if (r->callback_error != 0)
+        return r->callback_error;
+
+    return code;
 }
 
 static FAPI_retcode get_decodingID(FAPI_server *server, FAPI_font *ff, const char **decodingID_result)
@@ -1145,7 +1180,7 @@ static void release_glyphs(fapi_ufst_server *r, ufst_common_font_data *d)
         while (d->glyphs != 0) {
             pcleo_glyph_list_elem *e = d->glyphs;
             d->glyphs = e->next;
-            r->client_mem.free(&r->client_mem, e, "PCLEO char");
+            gs_free(r->mem, e, 0, 0, "PCLEO char");
         }
         d->glyphs = NULL;
     }
@@ -1153,6 +1188,40 @@ static void release_glyphs(fapi_ufst_server *r, ufst_common_font_data *d)
 
 static FAPI_retcode get_fontmatrix(FAPI_server *I, gs_matrix *m)
 {
+    fapi_ufst_server *r = If_to_I(I);
+    ufst_common_font_data *d = (ufst_common_font_data *)I->ff.server_font_data;
+    FAPI_retcode code = 0;
+    
+    if (d == 0) {
+        if ((code = make_font_data(r, I->ff.font_file_path, &(I->ff), &d)) != 0)
+            return (code);
+        I->ff.server_font_data = d;
+        prepare_typeface(r, d);
+    }
+    
+    /* There are PS jobs that rely on the standard fonts having
+     * a FontMatrix of [0.001 0 0 0.001 0 0], but MT fonts actually
+     * have an identity matrix. We need to compensate here. Other
+     * fonts need an identity matrix returned here, as we apply the
+     * font matrix explicitly in the scale calculation in zfapi.c
+     */
+    if (d->font_type & FC_FCO_TYPE) {
+        m->xx = 0.001;
+        m->xy = 0.0;
+        m->yx = 0.0;
+        m->yy = 0.001;
+        m->tx = 0.0;
+        m->ty = 0.0;
+    }
+    else {
+        m->xx = 1.0;
+        m->xy = 0.0;
+        m->yx = 0.0;
+        m->yy = 1.0;
+        m->tx = 0.0;
+        m->ty = 0.0;
+    }
+
 #if 0
     gs_matrix *base_font_matrix = &I->initial_FontMatrix;
     m->xx = I->initial_FontMatrix.xx;
@@ -1161,44 +1230,11 @@ static FAPI_retcode get_fontmatrix(FAPI_server *I, gs_matrix *m)
     m->yy = I->initial_FontMatrix.yy;
     m->tx = I->initial_FontMatrix.tx;
     m->ty = I->initial_FontMatrix.ty;
-#else
-    m->xx = 1;
-    m->xy = 0;
-    m->yx = 0;
-    m->yy = 1;
-    m->tx = 0;
-    m->ty = 0;
 #endif
-    return 0;
+    return (code);
 
 }
 
-static FAPI_retcode get_char_width(FAPI_server *server, FAPI_font *ff, FAPI_char_ref *c, FAPI_metrics *metrics)
-{   fapi_ufst_server *r = If_to_I(server);
-    UW16 buffer[2];
-    UW16 cc = (UW16)c->char_code;
-    WIDTH_LIST_INPUT_ENTRY li[1];
-    char PSchar_name[MAX_CHAR_NAME_LENGTH];
-    int code;
-    FSA_FROM_SERVER;
-
-#if !UFST_REENTRANT
-    static_server_ptr_for_ufst_callback = r;
-#endif
-    make_asciiz_char_name(PSchar_name, sizeof(PSchar_name), c);
-    r->ff = ff;
-    CGIFchIdptr(FSA &cc, PSchar_name);
-    li[0]. CharType.IF_cgnum = cc;
-    if ((code = CGIFwidth(FSA li, 1, 4, buffer)) != 0)
-        return code;
-    r->ff = 0;
-/*    CGIFhdr_font_purge (&r->fc);
-    release_glyphs(r, (ufst_common_font_data *)ff->server_font_data); */
-    metrics->escapement = buffer[0];
-    metrics->em_x = metrics->em_y = buffer[1];
-    return 0;
-}
-
 static int export_outline(fapi_ufst_server *r, PIFOUTLINE pol, FAPI_path *p)
 {   POUTLINE_CHAR outchar;
     SW16 num_contrs,num_segmts;
@@ -1265,7 +1301,7 @@ static FAPI_retcode get_char(fapi_ufst_server *r, FAPI_font *ff, FAPI_char_ref *
     ufst_common_font_data *d = (ufst_common_font_data *)ff->server_font_data;
     SW16 design_escapement[2];
     SW16 du_emx, du_emy;
-    char *notdef = ".notdef";
+    const char *notdef = ".notdef";
     const void *client_char_data = ff->char_data;
     const int client_char_data_len = ff->char_data_len;
     bool need_decrypt = ff->need_decrypt;
@@ -1343,12 +1379,12 @@ static FAPI_retcode get_char(fapi_ufst_server *r, FAPI_font *ff, FAPI_char_ref *
         r->callback_error = 0;
         ff->char_data = (void *)notdef;
         ff->char_data_len = strlen(notdef);
-        CGIFchIdptr(FSA &c1, notdef);
+        CGIFchIdptr(FSA (void *)&c1, (void *)notdef);
         code2 = CGIFchar_handle(FSA c1, &result, (SW16)0);
         if (code2 && code2 != ERR_fixed_space && code2 != ERR_bm_buff && code2 != ERR_bm_too_big) {
-            ff->char_data = "space";
+            ff->char_data = (void *)"space";
             ff->char_data_len = strlen("space");
-            CGIFchIdptr(FSA &c1, "space");
+            CGIFchIdptr(FSA (void *)&c1, (void *)"space");
             code2 = CGIFchar_handle(FSA c1, &result, (SW16)0);
         }
         if (!code2 || code2 == ERR_fixed_space) {
@@ -1493,19 +1529,36 @@ static FAPI_retcode get_char_outline(FAPI_server *server, FAPI_path *p)
 }
 
 static FAPI_retcode get_char_raster_metrics(FAPI_server *server, FAPI_font *ff, FAPI_char_ref *c, FAPI_metrics *metrics)
-{   fapi_ufst_server *r = If_to_I(server);
+{
+    fapi_ufst_server *r = If_to_I(server);
     int code;
 
     release_char_data_inline(r);
     code = get_char(r, ff, c, NULL, metrics, FC_BITMAP_TYPE);
     if (code == ERR_bm_buff || code == ERR_bm_too_big) /* Too big character ? */
-        return e_limitcheck;
+        return (e_VMerror);
     return code;
     /*	UFST cannot render enough metrics information without generating raster or outline.
         r->char_data keeps a raster after calling this function.
     */
 }
 
+static FAPI_retcode get_char_width(FAPI_server *server, FAPI_font *ff, FAPI_char_ref *c, FAPI_metrics *metrics)
+{
+    fapi_ufst_server *r = If_to_I(server);
+    int code;
+
+    release_char_data_inline(r);
+    code = get_char(r, ff, c, NULL, metrics, server->use_outline ? FC_CUBIC_TYPE : FC_BITMAP_TYPE);
+    if (code == ERR_bm_buff || code == ERR_bm_too_big) /* Too big character ? */
+        return (e_VMerror);
+    return code;
+    /*	UFST cannot render enough metrics information without generating raster or outline.
+        r->char_data keeps a raster after calling this function.
+    */
+}
+
+
 static FAPI_retcode get_char_raster(FAPI_server *server, FAPI_raster *rast)
 {   fapi_ufst_server *r = If_to_I(server);
 
@@ -1553,8 +1606,8 @@ static void release_fco(fapi_ufst_server *r, SW16 fcHandle)
 
             *e = ee->next;
             CGIFfco_Close(FSA ee->fcHandle);
-            r->client_mem.free(&r->client_mem, ee->file_path, "fco_file_path");
-            r->client_mem.free(&r->client_mem, ee, "fco_list_elem");
+            gs_free(r->mem, ee->file_path, 0, 0, "fco_file_path");
+            gs_free(r->mem, ee, 0, 0, "fco_list_elem");
         } else
             e = &(*e)->next;
 }
@@ -1579,11 +1632,11 @@ static FAPI_retcode FAPIU_release_typeface(FAPI_server *server, void *font_data)
 
     release_glyphs(r, d);
     release_fco(r, (SW16)(d->font_id >> 16));
-    r->client_mem.free(&r->client_mem, font_data, "ufst font data");
+    gs_free(r->mem, font_data, 0, 0, "ufst font data");
     return code;
 }
 
-static FAPI_retcode check_cmap_for_GID(FAPI_server *server, uint index)
+static FAPI_retcode check_cmap_for_GID(FAPI_server *server, uint *index)
 {
     return 0;
 }
@@ -1606,7 +1659,6 @@ static const FAPI_server If0 = {
     {0},
     0,
     false,
-    false,
     {1, 0, 0, 1, 0, 0},
     ensure_open,
     get_scaled_font,
@@ -1650,13 +1702,19 @@ int gs_fapiufst_instantiate(i_plugin_client_memory *client_mem, i_plugin_instanc
     i_ctx_t *i_ctx_p = NULL;
     gs_main_instance *inst;
     ref entry;
-    char tmppath[512];
+    char tmppath[gp_file_name_sizeof];
+    int code = 0;
 
     if (r == 0)
         return e_Fatal;
     memset(r, 0, sizeof(*r));
     r->If = If0;
-    r->client_mem = *client_mem;
+
+    code = gs_memory_chunk_wrap(&(r->mem), client_mem->client_data);
+    if (code != 0) {
+        return(code);
+    }
+
     *p_instance = &r->If.ig;
 
     inst = get_minst_from_memory((const gs_memory_t *)(client_mem->client_data));
@@ -1665,20 +1723,20 @@ int gs_fapiufst_instantiate(i_plugin_client_memory *client_mem, i_plugin_instanc
     /* The following entries will get overwritten if the user specifies alternative settings
      * on the command line.
      */
-    strncpy(tmppath, UFSTFONTDIR, sizeof(tmppath));
-    strncat(tmppath, UFST_PlugIn_Path, sizeof(tmppath));
+    strncpy(tmppath, UFSTFONTDIR, gp_file_name_sizeof);
+    strncat(tmppath, UFST_PlugIn_Path, gp_file_name_sizeof - strlen(tmppath));
 
     fapiu_make_string (i_ctx_p, (const char *)tmppath, &entry);
     (void)dict_put_string(systemdict, UFST_PlugIn_Str, &entry, NULL);
 
-    strncpy(tmppath, UFSTFONTDIR, sizeof(tmppath));
-    strncat(tmppath, FCOfontfilePath, sizeof(tmppath));
+    strncpy(tmppath, UFSTFONTDIR, gp_file_name_sizeof);
+    strncat(tmppath, FCOfontfilePath, gp_file_name_sizeof - strlen(tmppath));
 
     fapiu_make_string (i_ctx_p, (const char *)tmppath, &entry);
     (void)dict_put_string(systemdict, FCOfontfileStr, &entry, NULL);
 
-    strncpy(tmppath, UFSTFONTDIR, sizeof(tmppath));
-    strncat(tmppath, FCOfontfile2Path, sizeof(tmppath));
+    strncpy(tmppath, UFSTFONTDIR, gp_file_name_sizeof);
+    strncat(tmppath, FCOfontfile2Path, gp_file_name_sizeof - strlen(tmppath));
 
     fapiu_make_string (i_ctx_p, (const char *)tmppath, &entry);
     (void)dict_put_string(systemdict, FCOfontfile2Str, &entry, NULL);
@@ -1715,7 +1773,9 @@ static void gs_fapiufst_finit(i_plugin_instance *this, i_plugin_client_memory *m
     if (r->bInitialized && !r->ufst_is_singleton)
         gx_UFST_fini();
     if (r->param) {
-        mem->free (mem, r->param, "server_params");
+        gs_free (r->mem, r->param, 0, 0, "server_params");
     }
+    gs_memory_chunk_release (r->mem);
+
     mem->free(mem, r, "fapi_ufst_server");
 }
