@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2007 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Command line parsing and dispatching */
 #include "ctype_.h"
 #include "memory_.h"
@@ -173,6 +175,9 @@ gs_main_init_with_args(gs_main_instance * minst, int argc, char *argv[])
             } else if (!strcmp(argv[i], "--help")) {
                 print_help(minst);
                 helping = true;
+            } else if (!strcmp(argv[i], "--debug")) {
+                gs_debug_flags_list(minst->heap);
+                helping = true;
             } else if (!strcmp(argv[i], "--version")) {
                 print_version(minst);
                 puts(minst->heap, "");  /* \n */
@@ -290,8 +295,16 @@ run_stdin:
                 return code;
             break;
         case '-':               /* run with command line args */
+            if (strncmp(arg, "debug=", 6) == 0) {
+                code = gs_debug_flags_parse(minst->heap, arg+6);
+                if (code < 0)
+                    return code;
+                break;
+            }
+            /* FALLTHROUGH */
         case '+':
             pal->expand_ats = false;
+            /* FALLTHROUGH */
         case '@':               /* ditto with @-expansion */
             {
                 const char *psarg = arg_next(pal, &code);
@@ -305,27 +318,24 @@ run_stdin:
                 }
                 psarg = arg_copy(psarg, minst->heap);
                 if (psarg == NULL)
-                    return e_Fatal;
-                code = gs_main_init2(minst);
-                if (code < 0)
-                    return code;
-                code = run_string(minst, "userdict/ARGUMENTS[", 0);
-                if (code < 0)
-                    return code;
-                while ((arg = arg_next(pal, &code)) != 0) {
-                    char *fname = arg_copy(arg, minst->heap);
-                    if (fname == NULL)
-                        return e_Fatal;
-                    code = runarg(minst, "", fname, "", runInit);
-                    if (code < 0)
-                        return code;
-                }
-                if (code < 0)
-                    return e_Fatal;
-                code = runarg(minst, "]put", psarg, ".runfile", runInit | runFlush);
-                if (code < 0)
-                    return code;
-                return e_Quit;
+                    code = e_Fatal;
+                else
+                    code = gs_main_init2(minst);
+                if (code >= 0)
+                    code = run_string(minst, "userdict/ARGUMENTS[", 0);
+                if (code >= 0)
+                    while ((arg = arg_next(pal, &code)) != 0) {
+                        code = runarg(minst, "", arg, "", runInit);
+                        if (code < 0)
+                            break;
+                    }
+                if (code >= 0)
+                    code = runarg(minst, "]put", psarg, ".runfile", runInit | runFlush);
+                arg_free(psarg, minst->heap);
+                if (code >= 0)
+                    code = e_Quit;
+                
+                return code;
             }
         case 'A':               /* trace allocator */
             switch (*arg) {
@@ -366,16 +376,11 @@ run_stdin:
                     return code;
                 pal->expand_ats = false;
                 while ((arg = arg_next(pal, &code)) != 0) {
-                    char *sarg;
-
                     if (arg[0] == '@' ||
                         (arg[0] == '-' && !isdigit((unsigned char)arg[1]))
                         )
                         break;
-                    sarg = arg_copy(arg, minst->heap);
-                    if (sarg == NULL)
-                        return e_Fatal;
-                    code = runarg(minst, "", sarg, ".runstring", 0);
+                    code = runarg(minst, "", arg, ".runstring", 0);
                     if (code < 0)
                         return code;
                 }
@@ -448,7 +453,17 @@ run_stdin:
             return e_Info;      /* show usage info on exit */
         case 'I':               /* specify search path */
             {
-                char *path = arg_copy(arg, minst->heap);
+                const char *path;
+
+                if (arg[0] == 0) {
+                    path = arg_next(pal, &code);
+                    if (code < 0)
+                        return code;
+                } else
+                    path = arg;
+                if (path == NULL)
+                    return e_Fatal;
+                path = arg_copy(path, minst->heap);
                 if (path == NULL)
                     return e_Fatal;
                 gs_main_add_lib_path(minst, path);
@@ -731,19 +746,15 @@ static int
 argproc(gs_main_instance * minst, const char *arg)
 {
     int code = gs_main_init1(minst);            /* need i_ctx_p to proceed */
-    char *filearg;
 
     if (code < 0)
         return code;
-    filearg = arg_copy(arg, minst->heap);
-    if (filearg == NULL)
-        return e_Fatal;
     if (minst->run_buffer_size) {
         /* Run file with run_string. */
-        return run_buffered(minst, filearg);
+        return run_buffered(minst, arg);
     } else {
         /* Run file directly in the normal way. */
-        return runarg(minst, "", filearg, ".runfile", runInit | runFlush);
+        return runarg(minst, "", arg, ".runfile", runInit | runFlush);
     }
 }
 static int
@@ -801,7 +812,7 @@ runarg(gs_main_instance * minst, const char *pre, const char *arg,
         if (code < 0)
             return code;
     }
-    line = (char *)gs_alloc_bytes(minst->heap, len, "argproc");
+    line = (char *)gs_alloc_bytes(minst->heap, len, "runarg");
     if (line == 0) {
         lprintf("Out of memory!\n");
         return_error(e_VMerror);
@@ -812,6 +823,7 @@ runarg(gs_main_instance * minst, const char *pre, const char *arg,
     minst->i_ctx_p->starting_arg_file = true;
     code = run_string(minst, line, options);
     minst->i_ctx_p->starting_arg_file = false;
+    gs_free_object(minst->heap, line, "runarg");
     return code;
 }
 static int
@@ -904,6 +916,11 @@ static const char help_usage2[] = "\
  -sDEVICE=<devname>  select device         | -dBATCH  exit after last file\n\
  -sOutputFile=<file> select output file: - for stdout, |command for pipe,\n\
                                          embed %d or %ld for page #\n";
+#ifdef DEBUG
+static const char help_debug[] = "\
+ --debug=<option>[,<option>]*  select debugging options\n\
+ --debug                       list debugging options\n";
+#endif
 static const char help_trailer[] = "\
 For more information, see %s.\n\
 Please report bugs to bug-ghostscript@gnu.org.\n";
@@ -966,6 +983,9 @@ print_usage(const gs_main_instance *minst)
 {
     outprintf(minst->heap, "%s", help_usage1);
     outprintf(minst->heap, "%s", help_usage2);
+#ifdef DEBUG
+    outprintf(minst->heap, "%s", help_debug);
+#endif
 }
 
 /* compare function for qsort */
