@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Name lookup for Ghostscript interpreter */
 #include "memory_.h"
 #include "string_.h"
@@ -37,7 +39,7 @@ static const byte nt_1char_names[NT_1CHAR_SIZE] = {
     NT_1CHAR_NAMES_DATA
 };
 
-static void gs_names_finalize(void *);
+static void gs_names_finalize(const gs_memory_t *, void *);
 
 /* Structure descriptors */
 gs_private_st_simple(st_name_sub_table, name_sub_table, "name_sub_table");
@@ -130,10 +132,10 @@ names_init(ulong count, gs_ref_memory_t *imem)
 }
 
 static void
-gs_names_finalize(void *vptr)
+gs_names_finalize(const gs_memory_t *cmem, void *vptr)
 {
-    name_table *nt = vptr;
-    nt->memory->gs_lib_ctx->gs_name_table = NULL;
+    (void)vptr; /* unused */
+    cmem->gs_lib_ctx->gs_name_table = NULL;
 }
 
 /* Get the allocator for the name table. */
