@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Token scanner for Ghostscript interpreter */
 #include "ghost.h"
 #include "memory_.h"
@@ -592,13 +594,14 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
                         goto try_funny_name;
                     case '~':
                         s_A85D_init_inline(&sstate.s_ss.a85d);
-                        sstate.s_ss.st.template = &s_A85D_template;
+                        sstate.s_ss.st.templat = &s_A85D_template;
+                        sstate.s_ss.a85d.require_eod = true;
                         goto str;
                 }
                 scan_putback();
             }
             s_AXD_init_inline(&sstate.s_ss.axd);
-            sstate.s_ss.st.template = &s_AXD_template;
+            sstate.s_ss.st.templat = &s_AXD_template;
           str:scan_end_inline();
             dynamic_init(&da, imemory);
           cont_string:for (;;) {
@@ -606,7 +609,7 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
 
                 w.ptr = da.next - 1;
                 w.limit = da.limit - 1;
-                status = (*sstate.s_ss.st.template->process)
+                status = (*sstate.s_ss.st.templat->process)
                     (&sstate.s_ss.st, &s->cursor.r, &w,
                      s->end_status == EOFC);
                 if (!check_only)
@@ -664,7 +667,7 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
                 ((pstate->s_options & SCAN_FROM_STRING) != 0) &&
                 !scan_enable_level2;
             s_PSSD_partially_init_inline(&sstate.s_ss.pssd);
-            sstate.s_ss.st.template = &s_PSSD_template;
+            sstate.s_ss.st.templat = &s_PSSD_template;
             goto str;
         case '{':
             if (pstack == 0) {  /* outermost procedure */
@@ -703,6 +706,8 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
                           (pstack == pdepth ? 0 :
                           ref_stack_index(&o_stack, size)->value.intval),
                           size + pstack);
+                if (size > max_array_size)
+                    sreturn(e_limitcheck);
                 myref = (pstack == pdepth ? pref : &arr);
                 if (check_only) {
                     make_empty_array(myref, 0);
