@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 
 /* this is the ps interpreter interface to the jbig2decode filter
    used for (1bpp) scanned image compression. PDF only specifies
@@ -45,7 +47,7 @@
    a finalize method to deallocate it when the reference
    is no longer in use. */
 
-static void jbig2_global_data_finalize(void *vptr);
+static void jbig2_global_data_finalize(const gs_memory_t *cmem, void *vptr);
 gs_private_st_simple_final(st_jbig2_global_data_t, s_jbig2_global_data_t,
         "jbig2globaldata", jbig2_global_data_finalize);
 
@@ -119,9 +121,10 @@ z_jbig2makeglobalctx(i_ctx_t * i_ctx_p)
 }
 
 /* free our referenced global context data */
-static void jbig2_global_data_finalize(void *vptr)
+static void jbig2_global_data_finalize(const gs_memory_t *cmem, void *vptr)
 {
         s_jbig2_global_data_t *st = vptr;
+        (void)cmem; /* unused */
 
         if (st->data) s_jbig2decode_free_global_data(st->data);
         st->data = NULL;
